#!/usr/bin/env python
# -*- coding: utf-8 -*-
import json
from django.db import models
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.core.exceptions import ValidationError
from django.core.cache import cache
from django.contrib.sites.models import Site
from django.contrib.sites.managers import CurrentSiteManager
from django.contrib.redirects.models import Redirect
from django.utils import timezone
from django.utils.text import slugify

from .managers import PublishableManager
from .cache import _cache_key


class Date(models.Model):

    date_insert = models.DateTimeField(_(u"Date insert"), auto_now_add=True)
    date_update = models.DateTimeField(_(u"Date update"), auto_now=True)

    class Meta:
        abstract = True


class Owned(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL)

    class Meta:
        abstract = True


class OwnedNotRequired(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL,
                             blank=True, null=True)

    class Meta:
        abstract = True


class Publisher(Date):

    site = models.ForeignKey(Site, default=1)
    site_iid = models.PositiveIntegerField(
        _(u"Site id"),
        max_length=4,
        null=True, blank=True,
        db_index=True)
    site_domain = models.CharField(
        _(u"Site domain"),
        max_length=100,
        null=True, blank=True,
        db_index=True)
    mirror_site = models.ManyToManyField(
        'sites.Site',
        related_name="%(app_label)s_%(class)s_mirror_site",
        null=True, blank=True,
        verbose_name=_(u"Mirror site"),
    )
    date_available = models.DateTimeField(
        _(u"Date available"),
        default=timezone.now,
        null=True,
        db_index=True)
    published = models.BooleanField(
        _(u"Published"),
        default=False,
        db_index=True)

    objects = PublishableManager()
    on_site = CurrentSiteManager()

    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        self.site_domain = self.site.domain
        self.site_iid = self.site.id
        super(Publisher, self).save(*args, **kwargs)

    def is_published(self):
        return self.published and self.date_available <= timezone.now()


class Publishable(Owned, Publisher):
    class Meta:
        abstract = True


class NotUserPublishable(Publisher):
    class Meta:
        abstract = True


class Channeling(models.Model):

    channel = models.ForeignKey(
        'channels.Channel',
        verbose_name=_(u"Channel"),
    )
    mirror_channel = models.ManyToManyField(
        'channels.Channel',
        related_name="%(app_label)s_%(class)s_mirror_channel",
        null=True, blank=True,
        verbose_name=_(u"Mirror channel"),
    )
    channel_name = models.CharField(
        _(u"Channel name"),
        max_length=140,
        null=True, blank=True,
        db_index=True,
    )
    channel_long_slug = models.CharField(
        _(u"Channel long slug"),
        max_length=250,
        null=True, blank=True,
        db_index=True,
    )

    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        if self.channel:
            self.channel_name = self.channel.name
            self.channel_long_slug = self.channel.long_slug
        super(Channeling, self).save(*args, **kwargs)


class Slugged(models.Model):

    slug = models.SlugField(
        _(u"Slug"),
        db_index=True,
        max_length=150,
    )

    def clean(self):

        if self.slug in ("", None):
            try:
                self.slug = slugify(self.title)
            except:
                self.slug = slugify(self.name)
            else:
                pass

        self.validate_slug()

        if hasattr(self, 'get_absolute_url'):
            try:
                path = self.get_absolute_url()
            except:
                path = self.slug  # when get_absolute_url fails

            site = self.site or Site.objects.get(pk=1)
            redirect = Redirect.objects.filter(
                site=site,
                old_path=path
            )
            if redirect.exists():
                raise ValidationError(
                    _(u"The URL already exists as a redirect")
                )

        try:
            super(Slugged, self).clean()
        except AttributeError:
            pass  # does not implement the clean method

    def validate_slug(self):
        slug = getattr(self, 'slug', None)
        site = getattr(self, 'site', None)

        filters = {'slug': slug, 'site': site}

        if hasattr(self, 'channel'):
            filters['channel'] = self.channel

        if hasattr(self, 'parent'):
            filters['parent'] = self.parent

        # if model does not have site
        if not getattr(self, 'site', False):
            del filters['site']

        slug_exists = self.__class__.objects.filter(**filters)

        if getattr(self, 'pk', None):
            slug_exists = slug_exists.exclude(pk=self.pk)

        if settings.OPPS_SMART_SLUG_ENABLED:
            if slug_exists:
                last = slug_exists.latest('slug').slug
                suffix = last.split('-')[-1]
                if suffix.isdigit():
                    suffix = int(suffix) + 1
                    self.slug = "{0}-{1}".format(self.slug, suffix)
                else:
                    self.slug = "{0}-1".format(self.slug)
        else:
            if slug_exists.exists():
                raise ValidationError(_(u"Slug already exists."))

    def save(self, *args, **kwargs):
        if hasattr(self, 'get_absolute_url'):
            model = self.__class__
            if self.pk is not None:
                old_object = model.objects.get(pk=self.pk)
                if old_object.slug != self.slug:
                    redirect = Redirect(
                        site=self.site,
                        old_path=old_object.get_absolute_url(),
                        new_path=self.get_absolute_url()
                    )
                    redirect.save()

        super(Slugged, self).save(*args, **kwargs)

    class Meta:
        unique_together = ['site', 'slug']
        abstract = True


class Imaged(models.Model):
    main_image = models.ForeignKey(
        'images.Image',
        null=True, blank=True,
        on_delete=models.SET_NULL,
        related_name="%(app_label)s_%(class)s_mainimage",
        verbose_name=_(u'Main Image'))

    main_image_caption = models.CharField(
        _(u"Main image caption"),
        max_length=255,
        blank=True,
        null=True
    )

    images = models.ManyToManyField(
        'images.Image',
        null=True, blank=True,
        through='containers.ContainerImage')

    class Meta:
        abstract = True

    def all_images(self, check_published=True):
        cachekey = _cache_key(
            '{0}-all_images'.format(self.__class__.__name__),
            self.__class__, self.site_domain,
            u"{0}-{1}".format(self.channel_long_slug, self.slug))
        getcache = cache.get(cachekey)
        if getcache and check_published:
            return getcache

        imgs = []

        if self.main_image:
            self.main_image.caption = self.main_image_caption
            imgs.append(self.main_image)

        images = self.images.filter(
            date_available__lte=timezone.now()
        ).order_by('containerimage__order')
        if check_published:
            images = images.filter(published=True)

        if self.main_image:
            images = images.exclude(pk=self.main_image.pk)
        imgs += [i for i in images.distinct()]

        captions = dict(
            (ci.image_id, ci.caption) for ci in self.containerimage_set.all())

        if self.main_image:
            captions[self.main_image.id] = self.main_image.caption

        for im in imgs:
            caption = captions.get(im.pk)
            if caption:
                im.description = caption

        cache.set(cachekey, imgs)
        return imgs

    def get_thumb(self):
        return self.main_image


class Config(Publishable):
    """
    Basic key:value configuration for apps
    In admin it should be accessible only for users in developers group

    """

    FORMATS = (
        ('text', 'text'),
        ('json', 'json'),
        ('int', 'int'),
        ('float', 'float'),
        ('long', 'long'),
        ('comma', 'comma list'),
    )

    app_label = models.SlugField(
        _(u"App label"),
        db_index=True,
        max_length=150,
        null=True,
        blank=True
    )

    key_group = models.SlugField(
        _(u"Config Key Group"),
        db_index=True,
        max_length=150,
        null=True,
        blank=True
    )

    key = models.SlugField(
        _(u"Config Key"),
        db_index=True,
        max_length=150,
        unique=True,
    )

    format = models.CharField(_(u"Format"), max_length=20,
                              choices=FORMATS, default='text')
    value = models.TextField(_(u"Config Value"))
    description = models.TextField(_(u"Description"), blank=True, null=True)

    container = models.ForeignKey(
        'containers.Container',
        null=True, blank=True,
        help_text=_(u'Only published container'),
        on_delete=models.SET_NULL,
        verbose_name=_(u'Container')
    )
    channel = models.ForeignKey(
        'channels.Channel',
        null=True, blank=True,
        on_delete=models.SET_NULL,
        verbose_name=_(u'Channel')
    )

    class Meta:
        verbose_name = _(u'Config')
        verbose_name_plural = _(u'Configs')
        unique_together = ("app_label", "key_group", "key",
                           "site", "channel", "container")

    def __unicode__(self):
        return u"{0}-{1}".format(self.key, self.value)

    @classmethod
    def format_value(cls, value, format):
        casts = {
            'text': lambda value: value,
            'int': lambda value: int(value),
            'float': lambda value: float(value),
            'long': lambda value: long(value),
            'comma': lambda value: value.strip().split(','),
            'json': lambda value: json.loads(value),
        }
        return casts.get(format, lambda value: value)(value)

    def clean(self):
        try:
            Config.format_value(self.value, self.format)
        except:
            raise ValidationError(
                _(u"Can't format the value to %s") % self.format
            )

    @classmethod
    def get_keys(cls, **kwargs):
        keysqs = cls.objects.values('key')
        if kwargs:
            keysqs = keysqs.filter(**kwargs)
        return [item['key'] for item in keysqs]

    @classmethod
    def get_items(cls, **kwargs):
        itemsqs = cls.objects.values('key', 'value', 'format')
        if kwargs:
            itemsqs = itemsqs.filter(**kwargs)
        data = dict(
            [(item['key'], {
                'raw': item['value'],
                'format': item['format'],
                'value': cls.format_value(item['value'], item['format'])
            }) for item in itemsqs])
        return data

    @classmethod
    def get_value(cls, key, **kwargs):
        """
        kwargs must have filters to QuerySet
           site, channel, article, format, description
           return a single formated value
        """
        instance = cls.objects.filter(
            key=key,
            published=True,
            date_available__lte=timezone.now()
        )
        if kwargs:
            instance = instance.filter(**kwargs)

        if not instance:
            return False
        else:
            instance = instance.latest('date_insert')

        # format
        value = cls.format_value(instance.value, instance.format)

        return value

    @classmethod
    def get_values(cls, key_group, **kwargs):
        """
        kwargs must have filters to QuerySet
           site, channel, article, format, description
           return a dict of keys and formated values
        """
        instances = cls.objects.filter(
            key_group=key_group,
            published=True,
            date_available__lte=timezone.now()
        )
        if kwargs:
            instances = instances.filter(**kwargs)

        if not instances:
            return False

        value = {}
        for instance in instances:
            value[instance.key] = cls.format_value(instance.value,
                                                   instance.format)

        return value



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from opps.db.conf import settings

from redis import ConnectionPool
from redis import Redis as RedisClient


class Redis:
    def __init__(self, key_prefix, key_sufix, db=0):
        self.key_prefix = key_prefix
        self.key_sufix = key_sufix
        self.host = settings.OPPS_DB_HOST
        self.port = settings.OPPS_DB_PORT
        self.db = db

        pool = ConnectionPool(host=self.host,
                              port=self.port,
                              db=self.db)
        self.conn = RedisClient(connection_pool=pool)

    def __enter__(self):
        return self

    def __exit__(self, type, value, tb):
        self.close()

    def object(self):
        return self.conn

    def close(self):
        self.conn = None
        return True

    @property
    def key(self):
        return u'{0}_{1}_{2}'.format(settings.OPPS_DB_NAME,
                                     self.key_prefix,
                                     self.key_sufix).lower()

    def save(self, document):
        return self.conn.set(self.key, document)

    def publish(self, document):
        return self.conn.publish(self.key, document)

    def get(self):
        return self.conn.get(self.key)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.views.generic.detail import DetailView
from django.contrib.sites.models import get_current_site
from django.db.models import Q
from django.utils import timezone

from .models import FlatPage


class PageDetail(DetailView):

    model = FlatPage
    context_object_name = "context"
    type = 'flatpages'

    def get_template_names(self):
        _template_names = super(PageDetail, self).get_template_names()

        template_names = []

        base_path = '{}/{}.html'.format(self.type, self.page.get().slug)

        if self.site.id > 1:
            template_names.append('{}/{}'.format(self.site, base_path))

        template_names.append(base_path)

        return template_names + _template_names

    def get_queryset(self):
        self.site = get_current_site(self.request)
        self.slug = self.kwargs.get('slug')

        self.page = self.model.objects.filter(
            Q(global_page=True) | Q(site=self.site),
            slug=self.slug,
            date_available__lte=timezone.now(),
            published=True)
        return self.page



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from libthumbor import CryptoURL
from django.conf import settings


crypto = CryptoURL(key=settings.THUMBOR_SECURITY_KEY)


def _remove_prefix(url, prefix):
    if url.startswith(prefix):
        return url[len(prefix):]
    return url


def _remove_schema(url):
    return _remove_prefix(url, 'http://')


def _prepend_media_url(url):
    if url.startswith(settings.MEDIA_URL):
        url = _remove_prefix(url, settings.MEDIA_URL)
        url.lstrip('/')
        return u'{0}/{1}'.format(settings.THUMBOR_MEDIA_URL, url)
    return url


def image_url(image_url, **kwargs):
    if not settings.THUMBOR_ENABLED or image_url.split('.')[-1] == 'gif':
        # TODO: alternative ways for generating the thumbnails
        return image_url

    image_url = _prepend_media_url(image_url)
    image_url = _remove_schema(image_url)

    try:
        encrypted_url = crypto.generate(
            image_url=image_url,
            **dict(settings.THUMBOR_ARGUMENTS, **kwargs)).strip('/')
        return u'{0}/{1}'.format(settings.THUMBOR_SERVER, encrypted_url)
    except:
        return u""



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import random
import json

from django.views.decorators.csrf import csrf_exempt
from django.shortcuts import render, get_object_or_404
from django.http import HttpResponse
from django.utils import timezone
from django.utils.text import slugify
from django.contrib.auth.decorators import login_required
from django.contrib.sites.models import get_current_site

from opps.images.models import Image
from opps.containers.models import Container, ContainerImage
from opps.images.generate import image_url


def response_mimetype(request):
    if "application/json" in request.META['HTTP_ACCEPT']:
        return "application/json"
    return "text/plain"


@csrf_exempt
@login_required(login_url='/admin/')
def image_create(request, container_pk=None):

    if container_pk:
        container = get_object_or_404(Container, pk=int(container_pk))
        title = container.title
        site = container.site
        user = request.user or container.user
        date_available = container.date_available
    else:
        container = None
        title = 'Uploaded Image'
        site = get_current_site(request)
        user = request.user
        date_available = timezone.now()

    if request.method == "POST":
        f = request.FILES.get('image')

        title = request.POST.get('title') or container.title
        caption = request.POST.get('caption', '')
        source = request.POST.get('source', '')
        tags = request.POST.get('tags', '')
        slug = slugify(title)
        slug = "{0}-{1}".format(slug[:100], random.getrandbits(32))

        instance = Image(
            site=site,
            user=user,
            date_available=date_available,
            title=title,
            slug=slug,
            archive=f,
            source=source,
            published=True,
            tags=tags,
            description=caption
        )
        instance.save()

        if container:
            order = request.POST.get('order', 0)
            ContainerImage.objects.create(
                container=container,
                image=instance,
                caption=caption,
                order=int(order)
            )

        data = [{'name': f.name,
                 'url': "%s" % instance.archive.url,
                 'thumbnail_url': "%s" % image_url(
                     instance.archive.url,
                     width=60,
                     height=60
                 ),
                 "delete_url": "",
                 "delete_type": "DELETE"}]
        response = JSONResponse(data, {}, response_mimetype(request))
        response['Content-Disposition'] = 'inline; filename=files.json'
        return response
    else:
        return render(request, 'fileupload/image_form.html',
                      {'container': container})


class JSONResponse(HttpResponse):
    """JSON response class."""
    def __init__(self, obj='', json_opts={}, mimetype="application/json",
                 *args, **kwargs):
        content = json.dumps(obj, **json_opts)
        super(JSONResponse, self).__init__(content, mimetype, *args, **kwargs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.test import TestCase
from django.db import models
from django.contrib.auth import get_user_model
from django.contrib.sites.models import Site

from opps.containers.models import Container
from opps.containers.models import ContainerImage, ContainerBox
from opps.containers.models import ContainerBoxContainers, Mirror
from opps.containers.tasks import check_mirror_channel
from opps.channels.models import Channel


class ContainerModelTest(TestCase):

    def setUp(self):
        User = get_user_model()
        self.user = User.objects.create(username=u'test', password='test')
        self.site = Site.objects.filter(name=u'example.com').get()
        self.channel = Channel.objects.create(name=u'Home', slug=u'home',
                                              description=u'home page',
                                              site=self.site, user=self.user)

        self.container = Container.objects.create(title=u'test',
                                                  user=self.user,
                                                  published=True,
                                                  site=self.site,
                                                  channel=self.channel)

    def test_with_containers_exists(self):
        self.assertTrue(self.container)
        self.assertEqual(self.container.title, u'test')
        container = Container.objects.get(title=u'test')
        self.assertEqual(container, self.container)

    def test_save_with_test_desnomalization(self):
        self.assertEqual(self.container.channel_name, self.channel.name)
        self.assertEqual(self.container.channel_long_slug,
                         self.channel.long_slug)
        self.assertEqual(self.container.child_class, u"Container")
        self.assertEqual(self.container.child_module,
                         u"opps.containers.models")
        self.assertEqual(self.container.child_app_label, u"containers")

    def test_absolute_url(self):
        self.assertEqual(self.container.get_absolute_url(),
                         u"/home/test.html")

    def test_get_thumb(self):
        self.assertFalse(self.container.get_thumb())

    def test_search_category(self):
        self.assertEqual(self.container.search_category, u"Container")

    def test_get_http_absolute_url(self):
        self.assertEqual(self.container.get_http_absolute_url(),
                         u'http://example.com/home/test.html')

    def test_create_channel_mirror(self):
        channel = Channel.objects.create(name=u'Home2', slug=u'home2',
                                         description=u'home page2',
                                         site=self.site, user=self.user)
        self.container.mirror_channel.add(channel)
        self.container.save()
        check_mirror_channel(self.container, Mirror)
        mirror = Mirror.objects.all()

        self.assertTrue(mirror)
        self.assertEqual(len(mirror), 1)
        self.assertEqual(mirror[0].get_absolute_url(), u"/home2/test.html")
        self.assertEqual(len(Container.objects.all()), 2)

    def test_remove_channel_mirror(self):
        channel = Channel.objects.create(name=u'Home2', slug=u'home2',
                                         description=u'home page2',
                                         site=self.site, user=self.user)
        self.container.mirror_channel.add(channel)
        self.container.save()
        check_mirror_channel(self.container, Mirror)
        self.container.mirror_channel.remove(channel)
        self.container.save()
        check_mirror_channel(self.container, Mirror)
        mirror = Mirror.objects.all()

        self.assertFalse(mirror)
        self.assertEqual(len(mirror), 0)


class ContainerFields(TestCase):

    def test_title(self):
        field = Container._meta.get_field_by_name('title')[0]
        self.assertEqual(field.__class__, models.CharField)
        self.assertEqual(field.max_length, 140)
        self.assertTrue(field.db_index)

    def test_hat(self):
        field = Container._meta.get_field_by_name('hat')[0]
        self.assertEqual(field.__class__, models.CharField)
        self.assertEqual(field.max_length, 140)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)

    def test_short_url(self):
        field = Container._meta.get_field_by_name('short_url')[0]
        self.assertEqual(field.__class__, models.URLField)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)

    def test_child_class(self):
        field = Container._meta.get_field_by_name('child_class')[0]
        self.assertEqual(field.__class__, models.CharField)
        self.assertEqual(field.max_length, 30)
        self.assertTrue(field.db_index)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)

    def test_child_module(self):
        field = Container._meta.get_field_by_name('child_module')[0]
        self.assertEqual(field.__class__, models.CharField)
        self.assertEqual(field.max_length, 120)
        self.assertTrue(field.db_index)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)

    def test_child_app_label(self):
        field = Container._meta.get_field_by_name('child_app_label')[0]
        self.assertEqual(field.__class__, models.CharField)
        self.assertEqual(field.max_length, 30)
        self.assertTrue(field.db_index)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)

    def test_show_on_root_channel(self):
        field = Container._meta.get_field_by_name('show_on_root_channel')[0]
        self.assertEqual(field.__class__, models.BooleanField)
        self.assertTrue(field.default)

    def test_source(self):
        field = Container._meta.get_field_by_name('source')[0]
        self.assertEqual(field.__class__, models.CharField)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)


class ContainerImageFields(TestCase):

    def test_container(self):
        field = ContainerImage._meta.get_field_by_name(u"container")[0]
        self.assertEqual(field.__class__, models.ForeignKey)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)
        self.assertEqual(field.verbose_name, u"Container")

    def test_order(self):
        field = ContainerImage._meta.get_field_by_name(u"order")[0]
        self.assertEqual(field.__class__, models.PositiveIntegerField)
        self.assertEqual(field.default, 0)

    def test_image(self):
        field = ContainerImage._meta.get_field_by_name(u"image")[0]
        self.assertEqual(field.__class__, models.ForeignKey)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)
        self.assertEqual(field.verbose_name, u"Image")

    def test_caption(self):
        field = ContainerImage._meta.get_field_by_name(u"caption")[0]
        self.assertEqual(field.__class__, models.CharField)
        self.assertEqual(field.max_length, 255)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)


class ContainerBoxFields(TestCase):

    def test_title(self):
        field = ContainerBox._meta.get_field_by_name(u"title")[0]
        self.assertEqual(field.__class__, models.CharField)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)
        self.assertEqual(field.max_length, 140)

    def test_containers(self):
        field = ContainerBox._meta.get_field_by_name(u"containers")[0]
        self.assertEqual(field.__class__, models.ManyToManyField)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)
        self.assertEqual(field.verbose_name, u"Container")

    def test_queryset(self):
        field = ContainerBox._meta.get_field_by_name(u"queryset")[0]
        self.assertEqual(field.__class__, models.ForeignKey)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)
        self.assertEqual(field.verbose_name, u"Query Set")


class ContainerBoxContainersFields(TestCase):

    def test_containerbox(self):
        field = ContainerBoxContainers._meta.get_field_by_name(
            u"containerbox"
        )[0]
        self.assertEqual(field.__class__, models.ForeignKey)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)
        self.assertEqual(field.verbose_name, u"Container Box")

    def test_container(self):
        field = ContainerBoxContainers._meta.get_field_by_name(u"container")[0]
        self.assertEqual(field.__class__, models.ForeignKey)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)
        self.assertEqual(field.verbose_name, u"Container")

    def test_order(self):
        field = ContainerBoxContainers._meta.get_field_by_name(u"order")[0]
        self.assertEqual(field.__class__, models.PositiveIntegerField)
        self.assertEqual(field.default, 0)

    def test_aggregate(self):
        field = ContainerBoxContainers._meta.get_field_by_name(
            u"aggregate"
        )[0]
        self.assertEqual(field.default, 0)

    def test_date_available(self):
        field = ContainerBoxContainers._meta.get_field_by_name(
            u"date_available"
        )[0]
        self.assertEqual(field.__class__, models.DateTimeField)
        self.assertTrue(field.null)

    def test_date_end(self):
        field = ContainerBoxContainers._meta.get_field_by_name(u"date_end")[0]
        self.assertEqual(field.__class__, models.DateTimeField)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)

    def test_title(self):
        field = ContainerBoxContainers._meta.get_field_by_name(u"title")[0]
        self.assertEqual(field.__class__, models.CharField)
        self.assertEqual(field.max_length, 140)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)

    def test_hat(self):
        field = ContainerBoxContainers._meta.get_field_by_name(
            u"hat"
        )[0]
        self.assertEqual(field.__class__, models.CharField)
        self.assertEqual(field.max_length, 140)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)

    def test_url(self):
        field = ContainerBoxContainers._meta.get_field_by_name(
            u"url"
        )[0]
        self.assertEqual(field.__class__, models.CharField)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)

    def test_main_image(self):
        field = ContainerBoxContainers._meta.get_field_by_name(
            u"main_image"
        )[0]
        self.assertEqual(field.__class__, models.ForeignKey)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)
        self.assertEqual(field.verbose_name, u"Main Image")

    def test_main_image_caption(self):
        field = ContainerBoxContainers._meta.get_field_by_name(
            u"main_image_caption"
        )[0]
        self.assertEqual(field.__class__, models.CharField)
        self.assertEqual(field.max_length, 4000)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)
        self.assertEqual(field.help_text, u"Maximum characters 4000")



import djcelery

from django.conf import settings


def configure():
    if not settings.configured:

        test_settings = {
            'DATABASES': {
                'default': {
                    'ENGINE': 'django.db.backends.sqlite3',
                    'NAME': ':memory:',
                },
            },
            'INSTALLED_APPS': [
                'opps.contrib.admin',
                'opps.contrib.fileupload',
                'django.contrib.admin',
                'django.contrib.auth',
                'django.contrib.contenttypes',
                'django.contrib.sessions',
                'django.contrib.messages',
                'django.contrib.staticfiles',
                'django.contrib.sites',
                'django.contrib.redirects',

                'opps.core',
                'opps.core.tags',
                'opps.images',
                'opps.containers',
                'opps.boxes',
                'opps.channels',
                'opps.articles',
                'opps.sitemaps',
                'opps.flatpages',
                'opps.archives',
                'opps.views',
                'opps.fields',
                'opps.api',
                'opps.contrib.notifications',
            ],
            'TEMPLATE_CONTEXT_PROCESSORS': (
                'opps.channels.context_processors.channel_context',
            ),
            'MIDDLEWARE_CLASSES': (
                'django.contrib.sessions.middleware.SessionMiddleware',
                'django.middleware.common.CommonMiddleware',
                'django.middleware.csrf.CsrfViewMiddleware',
                'django.contrib.auth.middleware.AuthenticationMiddleware',
                'django.contrib.messages.middleware.MessageMiddleware',
                'django.middleware.clickjacking.XFrameOptionsMiddleware',
                'django.contrib.redirects.middleware.RedirectFallbackMiddleware',
            ),
            'HAYSTACK_CONNECTIONS': {
                'default': {
                    'ENGINE': 'haystack.backends.simple_backend.SimpleEngine',
                }
            },
            'OPPS_MULTISITE_FALLBACK': False,
            'ROOT_URLCONF': 'tests._site.urls',
            'STATIC_URL': '/static/',
            'ADMINS': ('admin@example.com',),
            'DEBUG': False,
            'SITE_ID': 1,
            'BROKER_URL': 'redis://localhost:6379/0',
            'CELERY_RESULT_BACKEND': 'redis://localhost:6379/0',
            'OPPS_MIRROR_CHANNEL': True,
            'OPPS_DB_HOST': '127.0.0.1',
            'OPPS_DB_PORT': 6379,
            'OPPS_DB_NAME': 'opps',
            'OPPS_DB_ENGINE': 'opps.db._redis.Redis',
        }

        djcelery.setup_loader()
        settings.configure(**test_settings)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.utils import timezone
from haystack.indexes import SearchIndex, CharField, DateTimeField, Indexable

from .models import Channel


class ChannelIndex(SearchIndex, Indexable):
    text = CharField(document=True, use_template=True)
    date_available = DateTimeField(model_attr='date_available')
    date_update = DateTimeField(model_attr='date_update')

    def get_model(self):
        return Channel

    def get_updated_field(self):
        return 'date_update'

    def index_queryset(self, using=None):
        return Channel.objects.filter(
            date_available__lte=timezone.now(),
            published=True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
INSTALL:
    apt-get install postgresql-contrib-9.1

RUN IN DBSHELL
   CREATE EXTENSION IF NOT EXISTS unaccent;

USAGE:

DATABASES = {
    'default': {
        'ENGINE': 'opps.db.backends.postgresql_psycopg2',
        ...
    }
}

SOUTH_DATABASE_ADAPTERS = {
    'default': 'south.db.postgresql_psycopg2'
}

DATABASE_POOL_ARGS = {
    'max_overflow': 10,
    'pool_size': 5,
    'recycle': 300
}
"""
import logging
from functools import partial

from sqlalchemy import event
from sqlalchemy.pool import manage, QueuePool
from psycopg2 import InterfaceError, ProgrammingError, OperationalError

from django.conf import settings
from django.db.backends.postgresql_psycopg2.base import *
from django.db.backends.postgresql_psycopg2.base import \
    DatabaseWrapper as Psycopg2DatabaseWrapper
from django.db.backends.postgresql_psycopg2.base import \
    CursorWrapper as DjangoCursorWrapper
from django.db.backends.postgresql_psycopg2.creation \
    import DatabaseCreation as Psycopg2DatabaseCreation


class DatabaseOperations(DatabaseOperations):
    def lookup_cast(self, lookup_type):
        if lookup_type in('icontains', 'istartswith'):
            return "UPPER(unaccent(%s::text))"
        else:
            return super(DatabaseOperations, self).lookup_cast(lookup_type)

"""
# Uncomment this class to disable the pool
class DatabaseWrapper(DatabaseWrapper):
    def __init__(self, *args, **kwargs):
        super(DatabaseWrapper, self).__init__(*args, **kwargs)
        self.operators['icontains'] = 'LIKE UPPER(unaccent(%s))'
        self.operators['istartswith'] = 'LIKE UPPER(unaccent(%s))'
        self.ops = DatabaseOperations(self)

# comment everything below to disable the pool
"""

POOL_SETTINGS = 'DATABASE_POOL_ARGS'

# DATABASE_POOL_ARGS should be something like:
# {'max_overflow':10, 'pool_size':5, 'recycle':300}
pool_args = getattr(settings, POOL_SETTINGS, {})
db_pool = manage(Database, **pool_args)

log = logging.getLogger('z.pool')


def _log(message, *args):
    log.debug(message)


# Only hook up the listeners if we are in debug mode.
if settings.DEBUG:
    event.listen(QueuePool, 'checkout', partial(_log, 'retrieved from pool'))
    event.listen(QueuePool, 'checkin', partial(_log, 'returned to pool'))
    event.listen(QueuePool, 'connect', partial(_log, 'new connection'))


def is_disconnect(e, connection, cursor):
    """
    Connection state check from SQLAlchemy:
    https://bitbucket.org/sqlalchemy/sqlalchemy/src/tip/
        lib/sqlalchemy/dialects/postgresql/psycopg2.py
    """
    if isinstance(e, OperationalError):
        # these error messages from libpq: interfaces/libpq/fe-misc.c.
        # TODO: these are sent through gettext in libpq and we can't
        # check within other locales - consider using connection.closed
        return 'terminating connection' in str(e) or \
               'closed the connection' in str(e) or \
               'connection not open' in str(e) or \
               'could not receive data from server' in str(e)
    elif isinstance(e, InterfaceError):
        # psycopg2 client errors, psycopg2/conenction.h, psycopg2/cursor.h
        return 'connection already closed' in str(e) or \
               'cursor already closed' in str(e)
    elif isinstance(e, ProgrammingError):
        # not sure where this path is originally from, it may
        # be obsolete.   It really says "losed", not "closed".
        return "losed the connection unexpectedly" in str(e)
    else:
        return False


class CursorWrapper(DjangoCursorWrapper):
    """
    A thin wrapper around psycopg2's normal cursor class so that we can catch
    particular exception instances and reraise them with the right types.

    Checks for connection state on DB API error and invalidates
    broken connections.
    """

    def __init__(self, cursor, connection):
        self.cursor = cursor
        self.connection = connection

    def execute(self, query, args=None):
        try:
            return self.cursor.execute(query, args)
        except Database.IntegrityError, e:
            raise utils.IntegrityError, utils.IntegrityError(
                *tuple(e)), sys.exc_info()[2]
        except Database.DatabaseError, e:
            if is_disconnect(e, self.connection.connection, self.cursor):
                log.error("invalidating broken connection")
                self.connection.invalidate()
            raise utils.DatabaseError, utils.DatabaseError(
                *tuple(e)), sys.exc_info()[2]

    def executemany(self, query, args):
        try:
            return self.cursor.executemany(query, args)
        except Database.IntegrityError, e:
            raise utils.IntegrityError, utils.IntegrityError(
                *tuple(e)), sys.exc_info()[2]
        except Database.DatabaseError, e:
            if is_disconnect(e, self.connection.connection, self.cursor):
                log.error("invalidating broken connection")
                self.connection.invalidate()
            raise utils.DatabaseError, utils.DatabaseError(
                *tuple(e)), sys.exc_info()[2]


class DatabaseCreation(Psycopg2DatabaseCreation):
    def destroy_test_db(self, *args, **kw):
        """Ensure conn pool is disposed before trying to drop database."""
        self.connection._dispose()
        super(DatabaseCreation, self).destroy_test_db(*args, **kw)


class DatabaseWrapper(Psycopg2DatabaseWrapper):
    """SQLAlchemy FTW."""

    def __init__(self, *args, **kwargs):
        super(DatabaseWrapper, self).__init__(*args, **kwargs)
        self.creation = DatabaseCreation(self)
        self.operators['icontains'] = 'LIKE UPPER(unaccent(%s))'
        self.operators['istartswith'] = 'LIKE UPPER(unaccent(%s))'
        self.ops = DatabaseOperations(self)

    def _cursor(self):
        if self.connection is None or self.connection.is_valid is False:
            self.connection = db_pool.connect(**self._get_conn_params())
            self.connection.set_client_encoding('UTF8')
            tz = 'UTC' if settings.USE_TZ else self.settings_dict.get(
                'TIME_ZONE')
            if tz:
                try:
                    get_parameter_status = self.connection.get_parameter_status
                except AttributeError:
                    # psycopg2 < 2.0.12 doesn't have get_parameter_status
                    conn_tz = None
                else:
                    conn_tz = get_parameter_status('TimeZone')

                if conn_tz != tz:
                    # Set the time zone in autocommit mode (see #17062)
                    self.connection.set_isolation_level(
                        psycopg2.extensions.ISOLATION_LEVEL_AUTOCOMMIT)
                    self.connection.cursor().execute(
                        self.ops.set_time_zone_sql(), [tz])
            self.connection.set_isolation_level(self.isolation_level)
            self._get_pg_version()
            connection_created.send(sender=self.__class__, connection=self)
        cursor = self.connection.cursor()
        cursor.tzinfo_factory = utc_tzinfo_factory if settings.USE_TZ else None
        return CursorWrapper(cursor, self.connection)

    def _commit(self):
        if self.connection is not None and self.connection.is_valid:
            return self.connection.commit()

    def _rollback(self):
        if self.connection is not None and self.connection.is_valid:
            return self.connection.rollback()

    def _dispose(self):
        """Dispose of the pool for this instance, closing all connections."""
        self.close()
        # _DBProxy.dispose doesn't actually call dispose on the pool
        conn_params = self._get_conn_params()
        key = db_pool._serialize(**conn_params)
        try:
            pool = db_pool.pools[key]
        except KeyError:
            pass
        else:
            pool.dispose()
            del db_pool.pools[key]

    def _get_conn_params(self):
        settings_dict = self.settings_dict
        if not settings_dict['NAME']:
            from django.core.exceptions import ImproperlyConfigured
            raise ImproperlyConfigured(
                "settings.DATABASES is improperly configured. "
                "Please supply the NAME value.")
        conn_params = {
            'database': settings_dict['NAME'],
        }
        conn_params.update(settings_dict['OPTIONS'])
        if 'autocommit' in conn_params:
            del conn_params['autocommit']
        if settings_dict['USER']:
            conn_params['user'] = settings_dict['USER']
        if settings_dict['PASSWORD']:
            conn_params['password'] = settings_dict['PASSWORD']
        if settings_dict['HOST']:
            conn_params['host'] = settings_dict['HOST']
        if settings_dict['PORT']:
            conn_params['port'] = settings_dict['PORT']
        return conn_params



# -*- coding: utf-8 -*-
from django.db import IntegrityError
from django.contrib import admin
from django.utils import timezone
from django.conf import settings
from django.contrib.sites.models import Site
from django.contrib.auth import get_user_model
from django.utils.translation import ugettext_lazy as _
from django.utils.text import slugify

from haystack.query import SearchQuerySet
from django.contrib.admin.views.main import ChangeList

from .models import Config
from .filters import ChildClassListFilter, ChannelListFilter

USE_HAYSTACK = getattr(settings, 'OPPS_ADMIN_USE_HAYSTACK', False)


class HaystackChangeList(ChangeList):
    def get_query_set(self, request):
        if not self.query:
            return super(HaystackChangeList, self).get_query_set(request)
        default_qs = self.root_query_set
        sqs = SearchQuerySet().models(self.model).auto_query(self.query)
        pks = [obj.pk for obj in sqs]
        self.root_query_set = self.root_query_set.filter(pk__in=pks)
        qs = super(HaystackChangeList, self).get_query_set(request)
        self.root_query_set = default_qs
        return qs


class HaystackModelAdmin(object):
    def get_changelist(self, request, **kwargs):
        if not USE_HAYSTACK:
            return super(
                HaystackModelAdmin, self
            ).get_changelist(request)
        return HaystackChangeList


class MassPublishMixin(admin.ModelAdmin):
    actions = ['publish']

    def publish(modeladmin, request, queryset):
        for obj in queryset:
            obj.published = not obj.published
            obj.save()
    publish.short_description = _(u'Publish/Unpublish')


class MassDuplicateMixin(admin.ModelAdmin):

    actions = ['duplicate']

    def duplicate(self, request, queryset):

        for obj in queryset:
            snapshot = {}
            remove_field = [
                'id', 'container_ptr', 'related_posts',
                'containerbox_containers', 'mirror_channel',
                'containers_mirror', 'images', 'postrelated_related',
                'postrelated_post', 'albums', 'mirror_site',
                'post_relatedposts', 'link_containers', 'slug',
                'polymorphic_ctype', 'date_insert', 'user']

            for key in obj._meta.get_all_field_names():
                try:
                    if key in remove_field:
                        continue

                    snapshot[key] = getattr(obj, key)
                    if snapshot[key] is None:
                        del snapshot[key]
                except AttributeError:
                    pass

            snapshot['published'] = False
            snapshot['user'] = request.user
            snapshot['title'] = "{0} - COPY :: {1}".format(snapshot['title'],
                                                           obj.pk)
            try:
                obj.__class__.objects.create(**snapshot)
            except IntegrityError:
                pass

    duplicate.short_description = _(u'Duplicate')


class PublisherAdmin(MassPublishMixin, MassDuplicateMixin):
    list_display = ['title', 'site', 'channel_long_slug',
                    'date_available', 'published', 'preview_url']
    list_filter = ['date_available', 'published', ChildClassListFilter]
    search_fields = ['title', 'slug', 'channel_name']

    def in_containerboxes(self, obj):
        articleboxes = obj.articlebox_articles.all()
        if articleboxes:
            html = [u"<ul>"]
            for box in articleboxes:
                li = (u"<li><a href='/admin/articles/articlebox/{box.id}/'"
                      u" target='_blank'>{box.slug}</a></li>")
                html.append(li.format(box=box))
            html.append(u"</ul>")
            return u"".join(html)
        return _(u"This item is not in a box")
    in_containerboxes.allow_tags = True
    in_containerboxes.short_description = _(u'Article boxes')

    def image_thumb(self, obj):
        if obj.main_image:
            return u'<img width="60px" height="60px" src="{0}" />'.format(
                obj.main_image.image_url(width=60, height=60)
            )
        return _(u'No Image')
    image_thumb.short_description = _(u'Thumbnail')
    image_thumb.allow_tags = True

    def images_count(self, obj):
        if obj.images:
            return obj.images.count()
        else:
            return 0
    images_count.short_description = _(u'Images')

    def preview_url(self, obj):
        html = (u'<a target="_blank" href="{href}" class="viewsitelink">'
                u'<i class="icon-eye-open icon-alpha75"></i>{text}</a>')
        return html.format(
            href=obj.get_absolute_url(),
            text=_(u"View on site")
        )
    preview_url.short_description = _(u"View on site")
    preview_url.allow_tags = True


class NotUserPublishableAdmin(PublisherAdmin):
    pass


class PublishableAdmin(PublisherAdmin):
    """
    Overrides standard admin.ModelAdmin save_model method
    It sets user (author) based on data from requet.
    """
    exclude = ('user',)

    def save_model(self, request, obj, form, change):
        if not change:
            obj.user = get_user_model().objects.get(pk=request.user.pk)
            obj.date_insert = timezone.now()
            if not obj.site:
                obj.site = Site.objects.get(pk=settings.SITE_ID)
        obj.date_update = timezone.now()
        obj.save()


class BaseBoxAdmin(PublishableAdmin):

    prepopulated_fields = {"slug": ["name"]}
    list_display = ['name', 'channel_name', 'date_available', 'published']
    list_filter = [ChannelListFilter, 'date_available', 'published', 'site']
    raw_id_fields = ['channel', 'article']
    search_fields = ['name', 'slug', 'channel_name']

    fieldsets = (
        (_(u'Identification'), {
            'fields': ('site', 'name', 'slug')}),
        (_(u'Relationships'), {
            'fields': (('channel', 'article'),)}),
        (_(u'Publication'), {
            'classes': ('extrapretty',),
            'fields': ('published', 'date_available')}),
    )

    def queryset(self, request):
        qs = super(BaseBoxAdmin, self).queryset(request)
        try:
            if not request.user.has_perm('containers.change_containerbox'):
                qs = qs.filter(queryset__isnull=True)
        except:
            pass  # admin model does not have the queryset field
        return qs


class ConfigAdmin(PublishableAdmin):
    list_display = ['key', 'app_label', 'key_group', 'channel', 'date_insert',
                    'date_available', 'published']
    list_filter = ["key", 'app_label', 'key_group', "channel", "published"]
    search_fields = ["key", "app_label", "key_group", "value"]

    raw_id_fields = ['channel', 'container']

admin.site.register(Config, ConfigAdmin)


def apply_rules(admin_class, app):
    """
    To allow overrides of admin rules for opps apps
    it uses the settings.py to load the values

    example of use:

    your project's settings.py

    OPPS_ADMIN_RULES = {
        'appname.ModelNameAdmin': {
            'fieldsets': (
                (u'Identification', {
                    'fields': ('site', 'title', 'slug')}),
            ),
            'list_display': (...),
            'list_filter': (...),
            'search_fields': (...),
            ...
        }
    }

    On appname/admin.py

    as a factory:

    from opps.core.admin import apply_rules
    ModelNameAdmin = apply_rules(ModelNameAdmin, 'appname')

    as a decorator:

    from opps.core.admin import apply_opps_rules

    @apply_opps_rules('appname')
    class ModelNameAdmin(admin.ModelAdmin):
        ...
    """

    key = "{0}.{1}".format(app, admin_class.__name__)
    OPPS_ADMIN_RULES = getattr(settings, 'OPPS_ADMIN_RULES', {})
    rules = OPPS_ADMIN_RULES.get(key)

    if not rules:
        return admin_class

    fieldsets = rules.get('fieldsets')
    if fieldsets:
        new_items = [(_(item[0]), item[1]) for item in fieldsets]
        admin_class.fieldsets = new_items

    attrs = ('list_display', 'list_filter',
             'search_fields', 'exclude', 'raw_id_fields',
             'prepopulated_fields', 'readonly_fields')

    for attr in attrs:
        to_apply = rules.get(attr)
        if to_apply:
            setattr(admin_class, attr, to_apply)

    field_overrides = rules.get('field_overrides')
    """
    Allow field attr overrides before form is rendered
    'images.ImagesAdmin': {
        'field_overrides': {
            "slug": {"help_text": "banana"}
        }
    }
    """
    if field_overrides:
        def get_form(self, request, obj=None, **kwargs):
            form = super(self.__class__, self).get_form(request, obj, **kwargs)
            if hasattr(form, 'base_fields'):
                for field, attrs in field_overrides.iteritems():
                    for attr, value in attrs.iteritems():
                        if isinstance(value, (str, unicode)):
                            value = _(value)
                        try:
                            setattr(form.base_fields[field], attr, value)
                        except:
                            pass  # KeyError base_fields[field]
            return form
        admin_class.get_form = get_form

    """
    Allow custom form for admin
    'articles.PostAdmin': {
        ...
        'form': 'yourapp.forms.PostAdminForm'
    },
    """
    form = rules.get('form')
    if form:
        try:
            _module = '.'.join(form.split('.')[:-1])
            _form = form.split('.')[-1]
            _temp = __import__(_module, globals(), locals(), [_form], -1)
            admin_class.form = getattr(_temp, _form)
        except:
            pass

    inlines = rules.get('inlines')
    if inlines:
        admin_class.inlines = []
        for inline in inlines:
            try:
                _module = '.'.join(inline.split('.')[:-1])
                _inline = inline.split('.')[-1]
                _temp = __import__(_module, globals(), locals(), [_inline], -1)
                admin_class.inlines.append(getattr(_temp, _inline))
            except:
                pass

    # TODO:
    # actions
    # override methods

    # load generic attributes
    specific_keys = list(attrs) + ['form', 'field_overrides',
                                   'fieldsets', 'inlines']
    for k, v in rules.iteritems():
        if k not in specific_keys:
            setattr(admin_class, k, v)

    return admin_class


def apply_opps_rules(app):

    def wrap(admin_class):
        admin_class = apply_rules(admin_class, app)
        return admin_class

    return wrap

apply_opps_rules.__doc__ = apply_rules.__doc__



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.contrib import admin
from django.contrib.sites.models import Site
from django.conf import settings
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
from django.contrib.admin.widgets import FilteredSelectMultiple
from .models import SitePermission


class AdminViewPermission(admin.ModelAdmin):

    site_lookup = 'site_iid__in'

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if settings.OPPS_MULTISITE_ADMIN and not request.user.is_superuser:
            sites_id = SitePermission.objects.filter(
                user=request.user,
                date_available__lte=timezone.now(),
                published=True
            ).values_list('site__id', flat=True)

            if db_field.name in ['site']:
                kwargs['queryset'] = Site.objects.filter(id__in=sites_id)

        result = super(AdminViewPermission, self).formfield_for_foreignkey(
            db_field, request, **kwargs
        )

        return result

    def queryset(self, request):
        qs = super(AdminViewPermission, self).queryset(request)
        if not settings.OPPS_MULTISITE_ADMIN or request.user.is_superuser:
            return qs

        sites_id = SitePermission.objects.filter(
            user=request.user,
            date_available__lte=timezone.now(),
            published=True
        ).values_list('site__id', flat=True)

        return qs.filter(**{self.site_lookup: sites_id})

    def get_form(self, request, obj=None, **kwargs):

        form = super(
            AdminViewPermission, self
        ).get_form(request, obj, **kwargs)

        sites_id = SitePermission.objects.filter(
            user=request.user,
            date_available__lte=timezone.now(),
            published=True
        ).values_list('site__id', flat=True)

        try:
            attrs = {}
            if settings.OPPS_MULTISITE_ADMIN and not request.user.is_superuser:
                qs = form.base_fields['mirror_site'].queryset
                form.base_fields['mirror_site'].queryset = qs.filter(
                    id__in=sites_id
                )
                attrs = {'disabled': 'disabled'}
            form.base_fields['mirror_site'].widget = FilteredSelectMultiple(
                _("Mirror site"), is_stacked=False, attrs=attrs
            )
        except:
            pass

        return form


class SitePermissionAdmin(admin.ModelAdmin):
    raw_id_fields = ('user',)
    exclude = ('site_iid', 'mirror_site', 'site_domain')


admin.site.register(SitePermission, SitePermissionAdmin)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models
from django.contrib.auth import get_user_model

User = get_user_model()


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'File.archive_link'
        db.add_column(u'archives_file', 'archive_link',
                      self.gf('django.db.models.fields.URLField')(max_length=255, null=True, blank=True),
                      keep_default=False)

        try:
            # Adding M2M table for field mirror_site on 'File'
            m2m_table_name = db.shorten_name(u'archives_file_mirror_site')
            db.create_table(m2m_table_name, (
                ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
                ('file', models.ForeignKey(orm[u'archives.file'], null=False)),
                ('site', models.ForeignKey(orm[u'sites.site'], null=False))
            ))
            db.create_unique(m2m_table_name, ['file_id', 'site_id'])
        except:
            pass


        # Changing field 'File.archive'
        db.alter_column(u'archives_file', 'archive', self.gf('django.db.models.fields.files.FileField')(max_length=255, null=True))

    def backwards(self, orm):
        # Deleting field 'File.archive_link'
        db.delete_column(u'archives_file', 'archive_link')

        try:
            # Removing M2M table for field mirror_site on 'File'
            db.delete_table(db.shorten_name(u'archives_file_mirror_site'))
        except:
            pass

        # Changing field 'File.archive'
        db.alter_column(u'archives_file', 'archive', self.gf('django.db.models.fields.files.FileField')(default='', max_length=255))

    models = {
        u'%s.%s' % (User._meta.app_label, User._meta.module_name): {
            'Meta': {'object_name': User.__name__},
        },
        u'archives.file': {
            'Meta': {'object_name': 'File'},
            'archive': ('django.db.models.fields.files.FileField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'archive_link': ('django.db.models.fields.URLField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'date_available': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'null': 'True', 'db_index': 'True'}),
            'date_insert': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'date_update': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'mirror_site': ('django.db.models.fields.related.ManyToManyField', [], {'blank': 'True', 'related_name': "u'archives_file_mirror_site'", 'null': 'True', 'symmetrical': 'False', 'to': u"orm['sites.Site']"}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'db_index': 'True'}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'default': '1', 'to': u"orm['sites.Site']"}),
            'site_domain': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'site_iid': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True', 'max_length': '4', 'null': 'True', 'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '150'}),
            'source': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'tags': ('django.db.models.fields.CharField', [], {'max_length': '4000', 'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '140', 'db_index': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['%s.%s']" % (User._meta.app_label, User._meta.object_name)})
        },
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'sites.site': {
            'Meta': {'ordering': "('domain',)", 'object_name': 'Site', 'db_table': "'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        }
    }

    complete_apps = ['archives']


#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.conf.urls import patterns, url
from django.conf import settings

from opps.core.cache import cache_page

from .views import AsyncServer, LongPullingServer


urlpatterns = patterns(
    '',
    url(r'^(?P<channel__long_slug>[\w//-]+)/(?P<slug>[\w-]+)\.server$',
        cache_page(settings.OPPS_CACHE_EXPIRE_DETAIL)(
            AsyncServer.as_view()), name='asyncserver'),
    url(r'^(?P<channel__long_slug>[\w//-]+)/(?P<slug>[\w-]+)\.pulingserver$',
        cache_page(settings.OPPS_CACHE_EXPIRE_DETAIL)(
            LongPullingServer.as_view()), name='pulingserver'),
)



from opps.core.managers import PublishableManager, PublishableQuerySet

from polymorphic.manager import PolymorphicManager
from polymorphic.query import PolymorphicQuerySet


class ContainerQuerySet(PolymorphicQuerySet, PublishableQuerySet):
    pass


class ContainerManager(PolymorphicManager, PublishableManager):
    queryset_class = ContainerQuerySet



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.conf import settings

from appconf import AppConf


class OppsDataBaseConf(AppConf):

    HOST = getattr(settings, 'OPPS_DB_HOSR', None)
    USER = getattr(settings, 'OPPS_DB_USER', None)
    PASSWORD = getattr(settings, 'OPPS_DB_PASSWORD', None)
    PORT = getattr(settings, 'OPPS_DB_PORT', None)
    NAME = getattr(settings, 'OPPS_DB_NAME', 'opps_db')
    ENGINE = getattr(settings, 'OPPS_DB_ENGINE', 'opps.db.')
    OPTION = getattr(settings, 'OPPS_BD_OPTION', None)

    class Meta:
        prefix = 'opps_db'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.core.exceptions import ValidationError
from django.conf import settings

from mptt.models import MPTTModel, TreeForeignKey
from mptt.managers import TreeManager

from opps.core.models import Publishable
from opps.core.models import Slugged


class ChannelManager(TreeManager):

    def get_homepage(self, site):
        try:
            return super(ChannelManager, self).get_query_set().filter(
                site__domain=site, homepage=True, published=True).get()
        except Channel.DoesNotExist:
            return None


class Channel(MPTTModel, Publishable, Slugged):

    name = models.CharField(_(u"Name"), max_length=60)
    long_slug = models.SlugField(_(u"Path name"), max_length=250,
                                 db_index=True)
    layout = models.CharField(_(u'Layout'), max_length=250, db_index=True,
                              default="default")
    description = models.CharField(_(u"Description"),
                                   max_length=255, null=True, blank=True)
    hat = models.CharField(_(u"Hat"),
                           max_length=255, null=True, blank=True)

    main_image = models.ForeignKey(
        'images.Image',
        null=True, blank=True,
        on_delete=models.SET_NULL,
        verbose_name=_(u'Main Image'),
    )

    show_in_menu = models.BooleanField(_(u"Show in menu?"), default=False)
    include_in_main_rss = models.BooleanField(
        _(u"Show in main RSS?"),
        default=True
    )
    homepage = models.BooleanField(
        _(u"Is home page?"),
        default=False,
        help_text=_(u'Check only if this channel is the homepage.'
                    u' Should have only one homepage per site')
    )
    group = models.BooleanField(_(u"Group sub-channel?"), default=False)
    order = models.IntegerField(_(u"Order"), default=0)
    parent = TreeForeignKey('self', related_name='subchannel',
                            null=True, blank=True,
                            verbose_name=_(u'Parent'))
    paginate_by = models.IntegerField(_("Paginate by"), null=True, blank=True)
    objects = ChannelManager()

    class Meta:
        unique_together = ("site", "long_slug", "slug", "parent")
        verbose_name = _(u'Channel')
        verbose_name_plural = _(u'Channels')
        ordering = ['name', 'parent__id', 'published']

    class MPTTMeta:
        unique_together = ("site", "long_slug", "slug", "parent")
        order_insertion_by = ['order']

    def __unicode__(self):
        """ Uniform resource identifier
        http://en.wikipedia.org/wiki/Uniform_resource_identifier
        """
        return u"/{0}/".format(self._set_long_slug())

    def get_absolute_url(self):
        return u"{0}".format(self.__unicode__())

    def get_thumb(self):
        return None

    def get_http_absolute_url(self):
        return u"http://{0}{1}".format(self.site_domain,
                                       self.get_absolute_url())
    get_http_absolute_url.short_description = _(u'Get HTTP Absolute URL')

    @staticmethod
    def autocomplete_search_fields():
        return ("id__iexact", "name__icontains", "long_slug__icontains", )

    @property
    def search_category(self):
        """for use in search result"""
        return _(u'Channel')

    @property
    def title(self):
        return self.name

    @property
    def root(self):
        return self.get_root()

    def clean(self):
        channel_is_home = Channel.objects.filter(
            site__id=settings.SITE_ID,
            homepage=True, published=True)
        if self.pk:
            channel_is_home = channel_is_home.exclude(pk=self.pk)

        if self.homepage and channel_is_home.exists():
            raise ValidationError('Exist home page!')

        super(Channel, self).clean()

    def _set_long_slug(self):
        if self.parent:
            return u"{0}/{1}".format(self.parent.long_slug, self.slug)
        return u"{0}".format(self.slug)

    def save(self, *args, **kwargs):
        self.long_slug = self._set_long_slug()
        super(Channel, self).save(*args, **kwargs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import celery
from django.utils import timezone


@celery.task
def check_mirror_site(container, Mirror):
    mirror_site = container.mirror_site.all()
    mirror_channel = container.mirror_channel.all()

    Mirror.objects.filter(
        container=container
    ).exclude(
        site__in=mirror_site
    ).exclude(
        channel__in=mirror_channel
    ).delete()

    for site in mirror_site:
        mirror, created = Mirror.objects.get_or_create(
            channel=container.channel,
            container=container,
            site=site,
            defaults=dict(
                container=container,
                user=container.user,
                title=container.title,
                published=container.published,
                slug=container.slug,
                channel_long_slug=container.channel.long_slug,
                channel_name=container.channel.name,
                main_image=container.main_image,
                hat=container.hat,
            )
        )

        if not created:
            mirror.title = container.title
            mirror.slug = container.slug
            mirror.published = container.published
            mirror.main_image = container.main_image
            mirror.channel_long_slug = container.channel.long_slug
            mirror.channel_name = container.channel.name
            mirror.site = site
            mirror.hat = container.hat
            mirror.save()


@celery.task
def check_mirror_channel(container, Mirror):
    mirror_channel = container.mirror_channel.all()

    if hasattr(container, "mirror_site"):
        mirror_site = container.mirror_site.all()
    else:
        mirror_site = []

    Mirror.objects.filter(
        container=container
    ).exclude(
        channel__in=mirror_channel
    ).exclude(
        site__in=mirror_site
    ).delete()

    for channel in mirror_channel:
        mirror, created = Mirror.objects.get_or_create(
            channel=channel,
            container=container,
            site=container.site,
            defaults=dict(
                container=container,
                user=container.user,
                title=container.title,
                published=container.published,
                slug=container.slug,
                channel_long_slug=channel.long_slug,
                channel_name=channel.name,
                main_image=container.main_image,
                hat=container.hat
            )
        )

        if not created:
            mirror.title = container.title
            mirror.slug = container.slug
            mirror.published = container.published
            mirror.main_image = container.main_image
            mirror.channel_long_slug = channel.long_slug
            mirror.channel_name = channel.name
            mirror.hat = container.hat
            mirror.save()


@celery.task.periodic_task(run_every=timezone.timedelta(minutes=30))
def check_all_mirror_channel():
    from .models import Container, Mirror

    mirror = Container.objects.all().exclude(mirror_channel=None)
    for obj in mirror:
        try:
            for channel in obj.mirror_channel:
                Mirror.objects.get(channel=channel, container=mirror,
                                   site=mirror.site)
        except:
            obj.save()



# -*- coding: utf-8 -*-
from urlparse import urlparse

from django.db import models
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
from django.core.exceptions import ValidationError
from django.core.cache import cache

from .signals import redirect_generate
from opps.containers.models import Container, ContainerImage
from opps.core.cache import _cache_key
from opps.core.managers import PublishableManager


class Article(Container):
    headline = models.TextField(
        _(u"Headline"),
        blank=True, null=True)
    short_title = models.CharField(
        _(u"Short title"),
        max_length=140,
        null=True, blank=True)

    class Meta:
        abstract = True


class Post(Article):
    content = models.TextField(_(u"Content"))
    albums = models.ManyToManyField(
        'articles.Album',
        null=True, blank=True,
        related_name='post_albums',
        verbose_name=_(u"Albums")
    )
    related_posts = models.ManyToManyField(
        'containers.Container',
        null=True, blank=True,
        related_name='post_relatedposts',
        through='articles.PostRelated',
    )

    objects = PublishableManager()

    class Meta:
        verbose_name = _('Post')
        verbose_name_plural = _('Posts')
        ordering = ['-date_available']

    def all_images(self, check_published=True):
        cachekey = _cache_key(
            '{0}main-all_images'.format(self.__class__.__name__),
            self.__class__, self.site_domain,
            u"{0}-{1}".format(self.channel_long_slug, self.slug))
        getcache = cache.get(cachekey)
        if getcache and check_published:
            return getcache

        imgs = super(Post, self).all_images()

        albums = self.albums.filter(date_available__lte=timezone.now())
        if check_published:
            albums = albums.filter(published=True)

        for album in albums:
            images = album.images.filter(
                date_available__lte=timezone.now(),
            ).exclude(
                pk__in=[i.pk for i in imgs]
            ).order_by('containerimage__order')
            if check_published:
                images = images.filter(published=True)

            captions = dict([
                (ci.image_id, ci.caption) for ci in
                ContainerImage.objects.filter(
                    container_id=album.pk
                )
            ])

            for image in images:
                caption = captions.get(image.pk)
                if caption:
                    image.description = caption
                imgs.append(image)

        cache.set(cachekey, imgs)
        return imgs

    def ordered_related(self, field='order'):
        """
        used in template
        """
        return self.related_posts.filter(
            published=True,
            date_available__lte=timezone.now()
        ).order_by(
            'postrelated_related__order'
        ).distinct()


class PostRelated(models.Model):

    post = models.ForeignKey(
        'articles.Post',
        verbose_name=_(u'Post'),
        null=True,
        blank=True,
        related_name='postrelated_post',
        on_delete=models.SET_NULL
    )
    related = models.ForeignKey(
        'containers.Container',
        verbose_name=_(u'Related Post'),
        null=True,
        blank=True,
        related_name='postrelated_related',
        on_delete=models.SET_NULL
    )
    order = models.PositiveIntegerField(_(u'Order'), default=0)

    class Meta:
        verbose_name = _('Related content')
        verbose_name_plural = _('Related contents')
        ordering = ('order',)

    def __unicode__(self):
        return u"{0}->{1}".format(self.related.slug, self.post.slug)


class Album(Article):
    class Meta:
        verbose_name = _('Album')
        verbose_name_plural = _('Albums')
        ordering = ['-date_available']


class Link(Article):
    url = models.URLField(_(u"URL"), null=True, blank=True)
    container = models.ForeignKey(
        'containers.Container',
        null=True, blank=True,
        related_name='link_containers'
    )

    class Meta:
        verbose_name = _('Link')
        verbose_name_plural = _('Links')
        ordering = ['-date_available']

    def is_local(self):
        try:
            url = urlparse(self.url)
            return url.netloc.replace('www', '') == self.site_domain
        except:
            return False

    def is_subdomain(self):
        return self.site_domain in self.url

    def clean(self):
        if not self.url and not self.container:
            raise ValidationError(_('URL field is required.'))

        self.url = self.url
        if self.container:
            self.url = self.container.get_absolute_url()


models.signals.post_save.connect(redirect_generate, sender=Link)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from unittest import TestCase
from django.http import HttpRequest

from opps.core.cache import cache_page


class DecoratorsTest(TestCase):

    def test_cache_page_new_style(self):
        """
        Test that we can call cache_page the new way
        """
        def my_view(request):
            return "response"

        my_view_cached = cache_page(123)(my_view)
        self.assertEqual(my_view_cached(HttpRequest()), "response")
        my_view_cached2 = cache_page(123, key_prefix="test")(my_view)
        self.assertEqual(my_view_cached2(HttpRequest()), "response")

    def test_cache_page_is_mobile(self):
        """
        Test that we can call cache_page in mobile page
        """
        def is_mobile_view(request):
            return request.is_mobile

        def mobile_view(request):
            return 'response'

        mobile_view_cached = cache_page(123)(mobile_view)
        is_mobile_view_cached = cache_page(123)(is_mobile_view)
        request = HttpRequest()
        request.is_mobile = True
        self.assertEqual(mobile_view_cached(request), "response")
        self.assertTrue(is_mobile_view_cached(request))



from django.core.management.base import BaseCommand, CommandError
from opps.channels.models import Channel
from opps.containers.models import ContainerBox
from opps.articles.models import Post


class Command(BaseCommand):

    def handle(self, *args, **options):
        models = [Channel, Post, ContainerBox]
        for m in models:
            [p.save() for p in m.objects.all()]



# -*- coding: utf-8 -*-

from haystack.indexes import \
    CharField, DateTimeField, MultiValueField, SearchIndex, IntegerField


class ContainerIndex(SearchIndex):
    text = CharField(document=True, use_template=True)
    channel_id = IntegerField(model_attr="channel_id")
    date_available = DateTimeField(model_attr='date_available')
    date_update = DateTimeField(model_attr='date_update')
    tags = MultiValueField(null=True)

    def prepare_tags(self, obj):
        if not obj.tags:
            return
        tags = []
        for tag in obj.get_tags() or []:
            tags.append(tag.slug)
            tags.append(tag.name)
        return tags

    def get_updated_field(self):
        return 'date_update'

    def index_queryset(self, using=None):
        return self.get_model().objects.all_published()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.conf import settings
from django.contrib.sites.models import get_current_site
from django.views.decorators.cache import cache_page as django_cache_page

from hashlib import md5


def _cache_key(_type, model, site, channel_long_slug):
    return md5((u'{0}:{1}:{2}:{3}:{4}'.format(
        _type,
        settings.OPPS_CACHE_PREFIX,
        model._meta.db_table,
        site,
        channel_long_slug)).replace(' ', '')).hexdigest()


def cache_page(*dec_args, **dec_kwargs):
    cache_timeout = dec_args[0] if dec_args else settings.OPPS_CACHE_EXPIRE
    cache_alias = dec_kwargs.pop('cache', None)
    key_prefix = dec_kwargs.pop('key_prefix', '')

    def decorator(func):
        def wrapped(*func_args, **func_kwargs):
            request = func_args[0]
            cache_prefix = u'{0}-{1}-{2}'.format(
                key_prefix,
                get_current_site(request).domain,
                getattr(request, 'is_mobile', False)
            )

            do_cache = django_cache_page(
                cache_timeout,
                cache=cache_alias,
                key_prefix=cache_prefix
            )(lambda *args, **kwargs: func(*func_args, **func_kwargs))

            return do_cache(*func_args, **func_kwargs)
        return wrapped
    return decorator



# -*- coding: utf-8 -*-

from django.http import Http404
from django.shortcuts import get_object_or_404
from django.utils import timezone
from django import template
from django.conf import settings
from django.contrib.sites.models import get_current_site

from opps.articles.models import Album
from opps.containers.models import Container, ContainerBox
from opps.channels.models import Channel


class View(object):

    context_object_name = "context"
    paginate_by = settings.OPPS_PAGINATE_BY
    limit = settings.OPPS_VIEWS_LIMIT
    page_kwarg = 'page'

    def __init__(self, *args, **kwargs):
        self.slug = None
        self.channel = None
        self.long_slug = None
        self.article = None
        self.child_class = u'container'
        self.channel_long_slug = []
        self.excluded_ids = set()
        super(View, self).__init__(*args, **kwargs)

    def get_paginate_by(self, queryset):
        queryset = self.get_queryset()

        setting_name = 'OPPS_{0}_{1}_PAGINATE_BY'.format(queryset.
                                                         model._meta.app_label,
                                                         queryset.model.
                                                         __name__).upper()

        by_settings = getattr(settings, setting_name, self.paginate_by)
        by_request = self.request.GET.get('paginate_by')
        by_channel = getattr(self.channel, 'paginate_by', None)

        return by_request or by_channel or by_settings

    def get_context_data(self, **kwargs):
        context = {}

        # channel is needed everywhere
        self.channel = self.channel or Channel.objects.get_homepage(
            site=get_current_site(self.request)
        )

        if not self.channel and getattr(
                settings, 'OPPS_MULTISITE_FALLBACK', None):
            self.channel = Channel.objects.filter(
                homepage=True, published=True)[:1].get()
            context['channel'] = self.channel

        if not self.long_slug:
            return context
        context = super(View, self).get_context_data(**kwargs)

        if hasattr(self, 'articleboxes'):
            context['articleboxes'] = self.articleboxes
        else:
            context['articleboxes'] = ContainerBox.objects.filter(
                channel__long_slug=self.long_slug)
            self.excluded_ids = []
            for box in context['articleboxes']:
                self.excluded_ids += [a.pk for a in box.ordered_containers()]

        obj_filter = {}
        obj_filter['site_domain'] = self.site.domain
        obj_filter['date_available__lte'] = timezone.now()

        obj_filter['published'] = True

        filters = obj_filter
        filters['channel_long_slug__in'] = self.channel_long_slug

        is_paginated = self.page_kwarg in self.request.GET
        if self.channel and self.channel.is_root_node() and not is_paginated:
            filters['show_on_root_channel'] = True
        article = Container.objects.filter(**filters)

        context['posts'] = article.filter(
            child_class='Post'
        ).exclude(pk__in=self.excluded_ids)[:self.limit]

        context['albums'] = Album.objects.filter(
            **filters
        ).exclude(pk__in=self.excluded_ids)[:self.limit]

        context['channel'] = {}
        context['channel']['long_slug'] = self.long_slug
        if self.channel:
            context['channel'] = self.channel

        context['breadcrumb'] = self.get_breadcrumb()

        if self.slug:
            try:
                context['next'] = self.get_object()\
                    .get_next_by_date_insert(**obj_filter)
            except self.get_object().DoesNotExist:
                pass
            try:
                context['prev'] = self.get_object()\
                    .get_previous_by_date_insert(**obj_filter)
            except self.get_object().DoesNotExist:
                pass

            context['articleboxes'] = context['articleboxes'].filter(
                containers__slug=self.slug)

            if self.get_object().child_class == 'Mirror':
                context['context'] = self.get_object().container

        if self.request.META.get('HTTP_X_PJAX', False) or\
           self.request.is_ajax():
            context['extends_parent'] = 'base_ajax.html'

        return context

    def get_template_folder(self):
        domain_folder = "containers"
        if self.site.id > 1:
            domain_folder = "{0}/containers".format(self.site.domain)
        return domain_folder

    def get_long_slug(self):
        self.long_slug = self.kwargs.get('channel__long_slug', None)
        try:
            if not self.long_slug:
                self.long_slug = Channel.objects.get_homepage(
                    site=self.site).long_slug
        except AttributeError:
            pass
        return self.long_slug

    def set_channel_rules(self):
        self.fallback = getattr(settings, 'OPPS_MULTISITE_FALLBACK', False)
        filters = dict(
            site__domain=self.site.domain,
            long_slug=self.long_slug,
            date_available__lte=timezone.now(),
            published=True
        )

        try:
            self.channel = Channel.objects.get(**filters)
        except Channel.DoesNotExist:
            if not self.fallback or self.site == self.site_master:
                raise Http404('Channel not found and fallback disabled')
            filters['site__domain'] = self.site_master.domain
            self.channel = get_object_or_404(Channel, **filters)

        self.long_slug = self.channel.long_slug

        self.channel_long_slug = [self.long_slug]
        self.channel_descendants = self.channel.get_descendants(
            include_self=False)
        for children in self.channel_descendants:
            self.channel_long_slug.append(children.long_slug)

    def get_breadcrumb(self):
        try:
            if self.channel.is_root_node():
                return []
        except:
            return []

        return self.channel.get_ancestors(include_self=True)

    def check_template(self, _template):
        try:
            template.loader.get_template(_template)
            return True
        except template.TemplateDoesNotExist:
            return False



# -*- encoding: utf-8 -*-
from django.utils import timezone
from django.contrib.sites.models import get_current_site
from django.conf import settings

from haystack.query import SearchQuerySet

from opps.views.generic.list import ListView
from opps.containers.models import Container

from opps.channels.models import Channel

from .models import Tag

USE_HAYSTACK = getattr(settings, 'OPPS_TAGS_USE_HAYSTACK', False)


class TagList(ListView):
    model = Container

    def get_template_list(self, domain_folder="containers"):
        templates = []

        list_name = 'list_tags'

        if self.request.GET.get('page') and\
           self.__class__.__name__ not in settings.OPPS_PAGINATE_NOT_APP:
            templates.append('{0}/{1}_paginated.html'.format(domain_folder,
                                                             list_name))

        templates.append('{0}/{1}.html'.format(domain_folder, list_name))
        return templates

    def get_context_data(self, **kwargs):
        context = super(TagList, self).get_context_data(**kwargs)
        context['tag'] = self.kwargs['tag']
        site = get_current_site(self.request)
        context['channel'] = Channel.objects.get_homepage(site)
        return context

    def get_queryset(self):
        self.site = get_current_site(self.request)
        # without the long_slug, the queryset will cause an error
        self.long_slug = 'tags'
        self.tag = self.kwargs['tag']

        if USE_HAYSTACK:
            return self.get_queryset_from_haystack()
        return self.get_queryset_from_db()

    def get_queryset_from_haystack(self):
        models = Container.get_children_models()
        sqs = SearchQuerySet().models(*models).filter(
            tags=self.tag).order_by('-date_available')
        sqs.model = Container
        return sqs

    def get_queryset_from_db(self):

        tags = Tag.objects.filter(slug=self.tag).values_list('name') or []
        tags_names = []
        if tags:
            tags_names = [i[0] for i in tags]

        ids = []
        for tag in tags_names:
            result = self.containers = self.model.objects.filter(
                site_domain=self.site,
                tags__contains=tag,
                date_available__lte=timezone.now(),
                published=True
            )
            if result.exists():
                ids.extend([i.id for i in result])

        # remove the repeated
        ids = list(set(ids))

        # grab the containers
        self.containers = self.model.objects.filter(id__in=ids)

        return self.containers



"""
Wrapper for loading templates from the filesystem based on the HTTP
request.

This template loader will use the thread-local value of
``template_dirs`` defined by the middlewares at
``opps.contrib.mobile.middleware`` to determine which template
directory to use.

"""

from django.conf import settings
from django.template.base import TemplateDoesNotExist
from django.template.loaders.filesystem import Loader as FileSystemLoader
from django.utils._os import safe_join

from opps.contrib.mobile.middleware import THREAD_LOCALS


class Loader(FileSystemLoader):

    def get_template_sources(self, template_name, template_dirs=None):
        if not template_dirs:
            template_dirs = getattr(
                THREAD_LOCALS,
                'template_dirs',
                settings.TEMPLATE_DIRS
            )
        for template_dir in template_dirs:
            try:
                yield safe_join(template_dir, template_name)
            except UnicodeDecodeError:
                # The template dir name was a bytestring that wasn't
                # valid UTF-8.
                raise
            except ValueError:
                # The joined path was located outside of this particular
                # template_dir (it might be inside another one, so this isn't
                # fatal).
                pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from django.core.files.images import get_image_dimensions

from .models import Image
from .forms import ImageModelForm
from opps.core.admin import PublishableAdmin
from opps.core.admin import apply_opps_rules
from opps.core.filters import UserListFilter


@apply_opps_rules('images')
class ImagesAdmin(PublishableAdmin):
    form = ImageModelForm
    prepopulated_fields = {"slug": ("title",)}
    list_display = ['image_thumb', 'image_dimension', 'title',
                    'date_available', 'published']
    list_filter = [UserListFilter, 'date_available', 'published']
    search_fields = ['title', 'slug']
    readonly_fields = ['image_thumb']
    exclude = ('user',)

    fieldsets = (
        (_(u'Identification'), {
            'fields': ('site', 'title', 'slug', 'archive', 'archive_link')}),
        (_(u'Crop'), {
            'fields': ('flip', 'flop', 'halign', 'valign', 'fit_in',
                       'smart', 'crop_x1', 'crop_x2', 'crop_y1', 'crop_y2',
                       'crop_example')}),
        (_(u'Content'), {
            'fields': ('description', 'tags', 'source')}),
        (_(u'Publication'), {
            'classes': ('extrapretty'),
            'fields': ('published', 'date_available')}),
    )

    def get_list_display(self, request):
        list_display = self.list_display
        pop = request.GET.get('pop')
        if pop == u'oppseditor':
            list_display = ['opps_editor_select'] + list(list_display)
        return list_display

    def opps_editor_select(self, obj):
        return u'''
        <a href="#" onclick="top.opps_editor_popup_selector('{0}')">{1}</a>
        '''.format(obj.image_url(), 'Select')
    opps_editor_select.short_description = _(u'Select')
    opps_editor_select.allow_tags = True

    def image_thumb(self, obj):
        if obj.archive or obj.archive_link:
            return u'<img width="60px" height="60px" src="{0}" />'.format(
                obj.image_url(width=60, height=60)
            )
        return _(u'No Image')
    image_thumb.short_description = _(u'Thumbnail')
    image_thumb.allow_tags = True

    def image_dimension(self, obj):
        if obj.archive:
            try:
                width, height = get_image_dimensions(obj.archive)
                return u"{0}x{1}".format(width, height)
            except:
                return ''
        return ''

    image_dimension.short_description = _(u'Dimension')

admin.site.register(Image, ImagesAdmin)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.contrib import admin

from .forms import FieldAdminForm
from .models import Field, Option, FieldOption


class FieldOptionRelatedInline(admin.TabularInline):
    model = FieldOption
    fk_name = 'field'
    raw_id_fields = ['option']
    actions = None
    ordering = ('order',)
    extra = 1
    classes = ('collapse',)


class FieldAdmin(admin.ModelAdmin):
    form = FieldAdminForm
    prepopulated_fields = {"slug":
                           ["application", "type", "name"]}
    inlines = [FieldOptionRelatedInline]


admin.site.register(Field, FieldAdmin)
admin.site.register(Option)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.conf.urls import patterns, url

from opps.core.cache import cache_page

from .views import PageDetail


urlpatterns = patterns(
    '',

    # FLATPAGEs
    url(r'^page/(?P<slug>[\w\b//-]+)/$',
        cache_page(60 * 2)(PageDetail.as_view()), name='open'),
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from .models import Post, PostRelated, Album, Link
from .forms import PostAdminForm, AlbumAdminForm, LinkAdminForm

from opps.containers.admin import ContainerAdmin, ContainerImageInline
from opps.core.admin import apply_opps_rules, HaystackModelAdmin
from opps.contrib.multisite.admin import AdminViewPermission


@apply_opps_rules('articles')
class PostRelatedInline(admin.TabularInline):
    model = PostRelated
    fk_name = 'post'
    raw_id_fields = ['related']
    actions = None
    ordering = ('order',)
    extra = 1
    classes = ('collapse',)
    verbose_name = _(u'Related post')
    verbose_name_plural = _(u'Related posts')


@apply_opps_rules('articles')
class PostAdmin(HaystackModelAdmin, ContainerAdmin, AdminViewPermission):

    form = PostAdminForm
    inlines = [ContainerImageInline, PostRelatedInline]
    search_fields = ['title', 'headline', 'slug', 'channel_name']
    raw_id_fields = ['main_image', 'channel',
                     'mirror_channel', 'albums']
    ordering = ('-date_available',)

    fieldsets = (
        (_(u'Identification'), {
            'fields': ('site', 'title', 'slug', 'get_http_absolute_url',
                       'short_url')}),
        (_(u'Content'), {
            'fields': ('hat', 'short_title', 'headline', 'content',
                       ('main_image', 'main_image_caption',
                        'image_thumb'), 'source', 'tags')}),
        (_(u'Custom'), {
            'fields': ('json',)}),
        (_(u'Relationships'), {
            'fields': ('channel', 'mirror_channel', 'albums',)}),
        (_(u'Publication'), {
            'classes': ('extrapretty'),
            'fields': ('published', 'date_available',
                       'show_on_root_channel', 'in_containerboxes')}),
    )


@apply_opps_rules('articles')
class AlbumAdmin(HaystackModelAdmin, ContainerAdmin, AdminViewPermission):

    form = AlbumAdminForm
    inlines = [ContainerImageInline]
    list_display = ['title', 'channel', 'images_count',
                    'date_available', 'published', 'preview_url']

    fieldsets = (
        (_(u'Identification'), {
            'fields': ('site', 'title', 'slug', 'get_http_absolute_url',
                       'short_url',)}),
        (_(u'Content'), {
            'fields': ('hat', 'short_title', 'headline',
                       ('main_image', 'main_image_caption',
                        'image_thumb'), 'tags')}),
        (_(u'Custom'), {
            'fields': ('json',)}),
        (_(u'Relationships'), {
            'fields': ('channel', 'mirror_channel')}),
        (_(u'Publication'), {
            'classes': ('extrapretty'),
            'fields': ('published', 'date_available',
                       'show_on_root_channel')}),
    )


@apply_opps_rules('articles')
class LinkAdmin(ContainerAdmin, AdminViewPermission):
    form = LinkAdminForm
    raw_id_fields = ['container', 'channel',
                     'mirror_channel', 'main_image']
    fieldsets = (
        (_(u'Identification'), {
            'fields': ('site', 'title', 'slug', 'get_http_absolute_url',
                       'short_url',)}),
        (_(u'Content'), {
            'fields': ('hat', 'short_title', 'headline', 'url', 'container',
                       ('main_image', 'main_image_caption',
                        'image_thumb'), 'tags')}),
        (_(u'Custom'), {
            'fields': ('json',)}),
        (_(u'Relationships'), {
            'fields': ('channel', 'mirror_channel')}),
        (_(u'Publication'), {
            'classes': ('extrapretty'),
            'fields': ('published', 'date_available',
                       'show_on_root_channel')}),
    )


admin.site.register(Post, PostAdmin)
admin.site.register(Album, AlbumAdmin)
admin.site.register(Link, LinkAdmin)



# -*- coding: utf-8 -*-

from django.db.models import Manager
from django.db import models
from django.utils import timezone


class PublishableQuerySet(models.query.QuerySet):

    def all_published(self):
        return self.filter(**self.get_all_published_lookups())

    def get_all_published_lookups(self, prefix=""):
        return {
            '%sdate_available__lte' % prefix: timezone.now(),
            '%spublished' % prefix: True,
        }


class PublishableManager(Manager):
    queryset_class = PublishableQuerySet

    def get_query_set(self):
        return self.queryset_class(self.model)

    def all_published(self):
        return self.get_query_set().all_published()

    def get_all_published_lookups(self, prefix=""):
        return self.get_query_set().get_all_published_lookups(prefix)



# -*- encoding: utf-8 -*-
from django.contrib import admin

from .models import Tag


class TagAdmin(admin.ModelAdmin):
    list_display = ('name', 'date_insert')
    search_fields = ('name',)
    prepopulated_fields = {"slug": ["name"]}

    fieldsets = [(None, {'fields': ('name', 'slug',)})]

    class Meta:
        model = Tag

admin.site.register(Tag, TagAdmin)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.test import TestCase
from django.db import models
from django.utils.translation import ugettext_lazy as _

from opps.core.models import Date, Publisher, Slugged, Channeling
from opps.core.models import Imaged, Config


class DateFields(TestCase):

    def test_date_insert(self):
        field = Date._meta.get_field_by_name('date_insert')[0]
        self.assertEqual(field.__class__, models.DateTimeField)
        self.assertTrue(field.auto_now_add)

    def test_date_update(self):
        field = Date._meta.get_field_by_name('date_update')[0]
        self.assertEqual(field.__class__, models.DateTimeField)
        self.assertTrue(field.auto_now)


class PublisherFields(TestCase):

    def test_site(self):
        field = Publisher._meta.get_field_by_name('site')[0]
        self.assertEqual(field.__class__, models.ForeignKey)

    def test_site_iid(self):
        field = Publisher._meta.get_field_by_name('site_iid')[0]
        self.assertEqual(field.__class__, models.PositiveIntegerField)
        self.assertEqual(field.max_length, 4)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)
        self.assertTrue(field.db_index)

    def test_site_domain(self):
        field = Publisher._meta.get_field_by_name('site_domain')[0]
        self.assertEqual(field.__class__, models.CharField)
        self.assertEqual(field.max_length, 100)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)
        self.assertTrue(field.db_index)

    def test_date_available(self):
        field = Publisher._meta.get_field_by_name('date_available')[0]
        self.assertEqual(field.__class__, models.DateTimeField)
        self.assertTrue(field.null)
        self.assertTrue(field.db_index)

    def test_published(self):
        field = Publisher._meta.get_field_by_name('published')[0]
        self.assertEqual(field.__class__, models.BooleanField)
        self.assertFalse(field.default)
        self.assertTrue(field.db_index)


class ChannelingFields(TestCase):

    def test_channel(self):
        field = Channeling._meta.get_field_by_name('channel')[0]
        self.assertEqual(field.__class__, models.ForeignKey)
        self.assertEqual(field.verbose_name, _(u"Channel"))

    def test_channel_name(self):
        field = Channeling._meta.get_field_by_name('channel_name')[0]
        self.assertEqual(field.__class__, models.CharField)
        self.assertEqual(field.max_length, 140)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)
        self.assertTrue(field.db_index)

    def test_channel_long_slug(self):
        field = Channeling._meta.get_field_by_name('channel_long_slug')[0]
        self.assertEqual(field.__class__, models.CharField)
        self.assertEqual(field.max_length, 250)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)
        self.assertTrue(field.db_index)


class SluggedTest(TestCase):

    def test_channel(self):
        field = Slugged._meta.get_field_by_name('slug')[0]
        self.assertEqual(field.__class__, models.SlugField)
        self.assertTrue(field.db_index)
        self.assertEqual(field.max_length, 150)


class ImagedTest(TestCase):

    def test_main_image(self):
        field = Imaged._meta.get_field_by_name('main_image')[0]
        self.assertEqual(field.__class__, models.ForeignKey)
        self.assertEqual(field.verbose_name, u'Main Image')
        self.assertTrue(field.blank)
        self.assertTrue(field.null)

    def test_main_image_caption(self):
        field = Imaged._meta.get_field_by_name('main_image_caption')[0]
        self.assertEqual(field.__class__, models.CharField)
        self.assertEqual(field.max_length, 255)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)

    def test_images(self):
        field = Imaged._meta.get_field_by_name('images')[0]
        self.assertEqual(field.__class__, models.ManyToManyField)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)


class ConfigTest(TestCase):

    def test_app_label(self):
        field = Config._meta.get_field_by_name('app_label')[0]
        self.assertEqual(field.__class__, models.SlugField)
        self.assertEqual(field.max_length, 150)
        self.assertTrue(field.db_index)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)

    def test_key_group(self):
        field = Config._meta.get_field_by_name('app_label')[0]
        self.assertEqual(field.__class__, models.SlugField)
        self.assertEqual(field.max_length, 150)
        self.assertTrue(field.db_index)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)

    def test_app_key(self):
        field = Config._meta.get_field_by_name('key')[0]
        self.assertEqual(field.__class__, models.SlugField)
        self.assertEqual(field.max_length, 150)
        self.assertTrue(field.unique)

    def test_format(self):
        field = Config._meta.get_field_by_name('format')[0]
        self.assertEqual(field.__class__, models.CharField)
        self.assertEqual(field.max_length, 20)
        self.assertEqual(field.default, 'text')

    def test_value(self):
        field = Config._meta.get_field_by_name('value')[0]
        self.assertEqual(field.__class__, models.TextField)

    def test_description(self):
        field = Config._meta.get_field_by_name('description')[0]
        self.assertEqual(field.__class__, models.TextField)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)

    def test_container(self):
        field = Config._meta.get_field_by_name('container')[0]
        self.assertEqual(field.__class__, models.ForeignKey)
        self.assertEqual(field.help_text, _(u'Only published container'))
        self.assertTrue(field.null)
        self.assertTrue(field.blank)

    def test_channel(self):
        field = Config._meta.get_field_by_name('channel')[0]
        self.assertEqual(field.__class__, models.ForeignKey)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.conf import settings

from appconf import AppConf


class OppsAPIConf(AppConf):

    NAME = getattr(settings, 'OPPS_API_NAME', u'v1')

    class Meta:
        prefix = 'opps_api'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.test import TestCase
from django.db import models

from opps.flatpages.models import FlatPage


class FlatPagesFields(TestCase):

    def test_show_in_menu(self):
        field = FlatPage._meta.get_field_by_name('show_in_menu')[0]
        self.assertFalse(field.default)

    def test_content(self):
        field = FlatPage._meta.get_field_by_name('content')[0]
        self.assertEqual(field.__class__, models.TextField)

    def test_order(self):
        field = FlatPage._meta.get_field_by_name('order')[0]
        self.assertEqual(field.default, 0)



from django.conf.urls import patterns
from .views import image_create

urlpatterns = patterns(
    '',
    (r'^image/(?P<container_pk>\d+)/$', image_create, {}, 'upload-new'),
    (r'^image/$', image_create, {}, 'upload-new-images'),
)



# -*- coding: utf-8 -*-
from django.contrib.sites.models import Site
from django.contrib.sites.models import get_current_site
from django.conf import settings

from opps.channels.models import Channel


class URLMiddleware(object):
    def process_request(self, request):
        """
        if the requested site is id 2 it
        will force the ROOT_URLCONF = 'yourproject.urls_2.py'
        """
        self.request = request
        site = get_current_site(request)
        if site.id > 1:
            prefix = "_{0}".format(site.id)
            self.request.urlconf = settings.ROOT_URLCONF + prefix


class TemplateContextMiddleware(object):
    """
    Include aditional items in response context_data
    """
    def process_template_response(self, request, response):
        if hasattr(response, 'context_data'):
            if 'channel' not in response.context_data:
                site = get_current_site(request)
                response.context_data['channel'] = Channel.objects\
                        .get_homepage(site=site or Site.objects.get(pk=1))
        return response



# -*- coding: utf-8 -*-
from django.shortcuts import render
from django.contrib.auth.decorators import login_required
from django.contrib.auth import get_user_model
from django.db.models import Q
from django.views.generic import ListView, FormView
from django.utils.decorators import method_decorator

from .forms import PopUpImageForm
from .models import Image


class PopUpImageView(FormView):
    form_class = PopUpImageForm
    template_name = 'images/image_add.html'

    def get_form_kwargs(self):
        kwargs = super(PopUpImageView, self).get_form_kwargs()
        kwargs['user'] = self.request.user
        return kwargs

    def form_valid(self, form):
        instance = form.save()
        return render(self.request, 'images/popup_done.html',
                      {'image': instance})


class GetImagesView(ListView):
    model = Image
    context_object_name = 'images'

    def get_template_names(self):
        if self.request.GET.get('template_name'):
            return ['images/{0}'.format(self.request.GET.get('template_name'))]
        return ['images/all_images.html']

    @method_decorator(login_required)
    def dispatch(self, *args, **kwargs):
        return super(GetImagesView, self).dispatch(*args, **kwargs)

    def get_queryset(self):
        queryset = super(GetImagesView, self).get_queryset().order_by('-id')

        query = Q()
        q = self.request.GET.get("q")
        user = self.request.GET.get("user") or 0
        published = self.request.GET.get("published", 1)

        if q:
            query &= Q(title__icontains=q) | Q(description__icontains=q)

        if user:
            query &= Q(user__id=user)

        if published:
            query &= Q(published=int(published))

        queryset = queryset.filter(query)
        return queryset._clone()

    def get_context_data(self, *args, **kwargs):
        q = self.request.GET.get("q")
        user = self.request.GET.get("user") or 0
        published = self.request.GET.get("published")

        context = {
            'users': get_user_model().objects.filter(
                image__isnull=False
            ).order_by('email').values('id', 'email').distinct(),
            'user_id': int(user),
            'published': published,
            'q': q,
        }
        context.update(kwargs)
        return super(GetImagesView, self).get_context_data(**context)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _

from opps.archives.models import Archive
from opps.core.tags.models import Tagged
from .generate import image_url as url

HALIGN_CHOICES = (
    ('left', _('Left')),
    ('center', _('Center')),
    ('right', _('Right'))
)
VALIGN_CHOICES = (
    ('top', _('Top')),
    ('middle', _('Middle')),
    ('bottom', _('Bottom'))
)


class Cropping(models.Model):
    crop_example = models.CharField(_(u"Crop Example"), max_length=255,
                                    null=True, blank=True)
    crop_x1 = models.PositiveSmallIntegerField(default=0, null=True,
                                               blank=True)
    crop_x2 = models.PositiveSmallIntegerField(default=0, null=True,
                                               blank=True)
    crop_y1 = models.PositiveSmallIntegerField(default=0, null=True,
                                               blank=True)
    crop_y2 = models.PositiveSmallIntegerField(default=0, null=True,
                                               blank=True)
    flip = models.BooleanField(_(u'Flip'), default=False,
                               help_text=_(u'Flag that indicates that '
                                           u'thumbor should flip '
                                           u'horizontally (on the vertical '
                                           u'axis) the image'))
    flop = models.BooleanField(_(u'Flop'), default=False,
                               help_text=_(u'Flag that indicates that '
                                           u'thumbor should flip '
                                           u'vertically (on the horizontal '
                                           u'axis) the image'))
    halign = models.CharField(_(u'Horizontal Align'), default=False,
                              max_length=6,
                              null=True, blank=True,
                              choices=HALIGN_CHOICES,
                              help_text=_(u'Horizontal alignment that '
                                          u'thumbor should use for cropping'))
    valign = models.CharField(_(u'Vertical Align'), default=False,
                              max_length=6,
                              null=True, blank=True,
                              choices=VALIGN_CHOICES,
                              help_text=_(u'Vertical alignment that '
                                          u'thumbor should use for cropping'))

    fit_in = models.BooleanField(_(u'Fit in'), default=False,
                                 help_text=_(u'Flag that indicates that '
                                             u'thumbor should fit the image '
                                             u'in the box defined by width x '
                                             u'height'))

    smart = models.BooleanField(_(u'Smart'), default=False,
                                help_text=_(u'Flag that indicates that'
                                            u' thumbor should use smart '
                                            u'cropping'))

    class Meta:
        abstract = True

    def clean(self):
        super(Cropping, self).clean()

    def save(self, *args, **kwargs):
        if self.archive and settings.THUMBOR_ENABLED:
            self.crop_example = self.archive.url
        else:
            self.crop_example = self.image_url()

        super(Cropping, self).save(*args, **kwargs)


class Image(Archive, Cropping, Tagged):

    def clean(self):
        items = ['x1', 'x2', 'y1', 'y2']
        for item in items:
            prop = getattr(self, 'crop_' + item, None)
            if not prop or prop is None or prop in ['', ' ']:
                setattr(self, 'crop_' + item, 0)

        if self.archive and settings.THUMBOR_ENABLED:
            self.crop_example = self.archive.url
        else:
            self.crop_example = self.image_url()

        super(Image, self).clean()

    def image_url(self, *args, **kwargs):
        if self.archive:
            return url(self.archive.url, *args, **kwargs)
        elif self.archive_link:
            return self.archive_link
        return ''

    class Meta:
        verbose_name = _(u'Image')
        verbose_name_plural = _(u'Images')



from django.test import TestCase

from opps.core.templatetags.obj_tags import ofKey


class OfKeyTest(TestCase):
    def test_tag(self):
        result = ofKey({"name": "andrews"}, "name")
        self.assertEqual(result, "andrews")

    def test_tag_is_none(self):
        result = ofKey(None, "name")
        self.assertEqual(result, "")



# -*- coding: utf-8 -*-
from django import template
from django.conf import settings
from django.core.cache import cache

from opps.channels.models import Channel


register = template.Library()


@register.assignment_tag
def get_channel(slug):
    """
        Usage:

        {% get_channel "videos" %}
    """
    try:
        return Channel.objects.get(site=settings.SITE_ID, slug=slug,
                                   published=True)
    except Channel.DoesNotExist:
        return Channel.objects.none()


@register.assignment_tag
def get_channels_by(**filters):
    """Return a list of channels filtered by given args"""
    cache_key = u'getchannelsby-{0}'.format(hash(frozenset(filters.items())))
    if cache.get(cache_key):
        return cache.get(cache_key)

    channels = Channel.objects.filter(site=settings.SITE_ID, published=True,
                                      **filters)
    cache.set(cache_key, channels, 60 * 60)
    return channels



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.contrib.sites.models import Site
from django.conf import settings


class DynamicSiteMiddleware(object):

    def hosting_parse(self, hosting):
        """
        Returns ``(host, port)`` for ``hosting`` of the form ``'host:port'``.

        If hosting does not have a port number, ``port`` will be None.
        """
        if ':' in hosting:
            return hosting.rsplit(':', 1)
        return hosting, None

    def get_hosting(self, hosting):
        domain, port = self.hosting_parse(hosting)
        if domain in settings.OPPS_DEFAULT_URLS:
            domain = 'example.com'
        try:
            return Site.objects.get(domain=domain)
        except Site.DoesNotExist:
            return Site.objects.order_by('id')[0]

    def process_request(self, request):
        hosting = request.get_host().lower()
        site = self.get_hosting(hosting)

        request.site = site
        settings.SITE_ID = site.id
        settings.CACHE_MIDDLEWARE_KEY_PREFIX = "opps_site-{0}".format(site.id)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import copy
import json
from django.db import models
from django.core.serializers.json import DjangoJSONEncoder
from django.utils.translation import ugettext_lazy as _
from django.utils import six
from django.forms import fields
from django.forms.util import ValidationError

from south.modelsinspector import add_introspection_rules

from .subclassing import SubfieldBase


class JSONFormFieldBase(object):

    def to_python(self, value):
        if isinstance(value, six.string_types):
            try:
                return json.loads(value)
            except ValueError:
                raise ValidationError(_("Enter valid JSON"))
        return value

    def clean(self, value):

        if not value and not self.required:
            return None

        # Trap cleaning errors & bubble them up as JSON errors
        try:
            return super(JSONFormFieldBase, self).clean(value)
        except TypeError:
            raise ValidationError(_("Enter valid JSON"))


class JSONFormField(JSONFormFieldBase, fields.Field):
    pass


class JSONCharFormField(JSONFormFieldBase, fields.CharField):
    pass


class JSONFieldBase(six.with_metaclass(SubfieldBase, models.Field)):

    def __init__(self, *args, **kwargs):
        self.dump_kwargs = kwargs.pop('dump_kwargs', {
            'cls': DjangoJSONEncoder,
            'separators': (',', ':')
        })
        self.load_kwargs = kwargs.pop('load_kwargs', {})

        super(JSONFieldBase, self).__init__(*args, **kwargs)

    def pre_init(self, value, obj):
        """Convert a string value to JSON only if it needs
        to be deserialized.

        SubfieldBase meteaclass has been modified to call
        this method instead of
        to_python so that we can check the obj state and
        determine if it needs to be
        deserialized"""

        if obj._state.adding:
            # Make sure the primary key actually exists on the object before
            # checking if it's empty. This is a special case for
            # South datamigrations
            # see: https://github.com/bradjasper/django-jsonfield/issues/52
            if not hasattr(obj, "pk") or obj.pk is not None:
                if isinstance(value, six.string_types):
                    try:
                        return json.loads(value, **self.load_kwargs)
                    except ValueError:
                        raise ValidationError(_("Enter valid JSON"))

        return value

    def to_python(self, value):
        """The SubfieldBase metaclass calls pre_init instead of to_python,
        however to_python
        is still necessary for Django's deserializer"""
        return value

    def get_db_prep_value(self, value, connection, prepared=False):
        """Convert JSON object to a string"""
        if self.null and value is None:
            return None
        return json.dumps(value, **self.dump_kwargs)

    def value_to_string(self, obj):
        value = self._get_val_from_obj(obj)
        return self.get_db_prep_value(value, None)

    def value_from_object(self, obj):
        value = super(JSONFieldBase, self).value_from_object(obj)
        if self.null and value is None:
            return None
        return self.dumps_for_display(value)

    def dumps_for_display(self, value):
        return json.dumps(value, **self.dump_kwargs)

    def formfield(self, **kwargs):

        if "form_class" not in kwargs:
            kwargs["form_class"] = self.form_class

        field = super(JSONFieldBase, self).formfield(**kwargs)

        if not field.help_text:
            field.help_text = "Enter valid JSON"

        return field

    def get_default(self):
        """
        Returns the default value for this field.

        The default implementation on models.Field calls force_unicode
        on the default, which means you can't set arbitrary Python
        objects as the default. To fix this, we just return the value
        without calling force_unicode on it. Note that if you set a
        callable as a default, the field will still call it. It will
        *not* try to pickle and encode it.

        """
        if self.has_default():
            if callable(self.default):
                return self.default()
            return copy.deepcopy(self.default)
        # If the field doesn't have a default, then we punt to models.Field.
        return super(JSONFieldBase, self).get_default()

    def db_type(self, connection):
        if connection.vendor == 'postgresql' and\
           connection.pg_version >= 90400:
            return 'json'
        else:
            return super(JSONFieldBase, self).db_type(connection)


class JSONField(JSONFieldBase, models.TextField):
    """JSONField is a generic textfield that serializes/unserializes
    JSON objects"""
    form_class = JSONFormField

    def dumps_for_display(self, value):
        kwargs = {"indent": 2}
        kwargs.update(self.dump_kwargs)
        return json.dumps(value, **kwargs)


class JSONCharField(JSONFieldBase, models.CharField):
    """JSONCharField is a generic textfield that serializes/unserializes
    JSON objects,
    stored in the database like a CharField, which enables it to be used
    e.g. in unique keys"""
    form_class = JSONCharFormField


add_introspection_rules([], ["^opps\.db\.models\.fields\.jsonf\."
                             "(JSONField|JSONCharField)"])



# -*- encoding: utf-8 -*-
from django.test import TestCase
from django.db import models

from opps.images.models import Cropping, HALIGN_CHOICES, VALIGN_CHOICES


class CroppingFields(TestCase):
    def test_crop_example(self):
        field = Cropping._meta.get_field_by_name('crop_example')[0]
        self.assertTrue(field.__class__, models.CharField)
        self.assertTrue(field.blank)
        self.assertTrue(field.null)

    def test_crop_x1(self):
        field = Cropping._meta.get_field_by_name('crop_x1')[0]
        self.assertTrue(field.__class__, models.PositiveSmallIntegerField)
        self.assertTrue(field.blank)
        self.assertTrue(field.null)

    def test_crop_x2(self):
        field = Cropping._meta.get_field_by_name('crop_x2')[0]
        self.assertTrue(field.__class__, models.PositiveSmallIntegerField)
        self.assertTrue(field.blank)
        self.assertTrue(field.null)

    def test_crop_y1(self):
        field = Cropping._meta.get_field_by_name('crop_y1')[0]
        self.assertTrue(field.__class__, models.PositiveSmallIntegerField)
        self.assertTrue(field.blank)
        self.assertTrue(field.null)

    def test_crop_y2(self):
        field = Cropping._meta.get_field_by_name('crop_y2')[0]
        self.assertTrue(field.__class__, models.PositiveSmallIntegerField)
        self.assertTrue(field.blank)
        self.assertTrue(field.null)

    def test_flip(self):
        field = Cropping._meta.get_field_by_name('flip')[0]
        self.assertTrue(field.__class__, models.BooleanField)
        self.assertFalse(field.default)

    def test_flop(self):
        field = Cropping._meta.get_field_by_name('flop')[0]
        self.assertTrue(field.__class__, models.BooleanField)
        self.assertFalse(field.default)

    def test_halign(self):
        field = Cropping._meta.get_field_by_name('halign')[0]
        self.assertTrue(field.__class__, models.CharField)
        self.assertFalse(field.default)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)
        self.assertEqual(field.choices, HALIGN_CHOICES)

    def test_valign(self):
        field = Cropping._meta.get_field_by_name('valign')[0]
        self.assertTrue(field.__class__, models.CharField)
        self.assertFalse(field.default)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)
        self.assertEqual(field.choices, VALIGN_CHOICES)

    def test_fit_in(self):
        field = Cropping._meta.get_field_by_name('fit_in')[0]
        self.assertTrue(field.__class__, models.BooleanField)
        self.assertFalse(field.default)

    def test_smart(self):
        field = Cropping._meta.get_field_by_name('smart')[0]
        self.assertTrue(field.__class__, models.BooleanField)
        self.assertFalse(field.default)



from django.conf.urls import patterns, include, url

from django.contrib import admin
admin.autodiscover()

urlpatterns = patterns('',
    # Examples:
    # url(r'^$', 'tests.views.home', name='home'),
    # url(r'^blog/', include('blog.urls')),

    url(r'^admin/', include(admin.site.urls)),
    url(r'^', include('opps.urls')),
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.contrib.sites.models import get_current_site
from django.utils import timezone
from django.conf import settings

from opps.views.generic.list import ListView
from opps.containers.views import ContainerList
from opps.containers.models import Container, ContainerBox
from opps.articles.models import Album


class AlbumList(ContainerList):
    model = Album
    type = 'articles'

    def get_template_names(self):
        templates = []

        domain_folder = self.get_template_folder()
        list_name = 'list'

        templates.append('{0}/{1}/{2}.html'.format(
            self.model._meta.app_label,
            self.model._meta.module_name, list_name))

        if self.request.GET.get('page') and\
           self.__class__.__name__ not in\
           settings.OPPS_PAGINATE_NOT_APP:
            templates.append('{0}/{1}/{2}/{3}_paginated.html'.format(
                domain_folder, self.model._meta.app_label,
                self.model._meta.module_name, list_name))

        return templates

    def get_queryset(self):
        # TODO: refatoring, used super()
        self.site = get_current_site(self.request)
        self.long_slug = self.get_long_slug()

        if not self.long_slug:
            return None

        self.set_channel_rules()

        self.articleboxes = ContainerBox.objects.filter(
            channel__long_slug=self.long_slug)

        is_paginated = self.page_kwarg in self.request.GET

        if not is_paginated:
            for box in self.articleboxes:
                self.excluded_ids.update(
                    [a.pk for a in box.ordered_containers()])

        filters = {}
        filters['site_domain'] = self.site.domain
        filters['date_available__lte'] = timezone.now()
        filters['published'] = True
        filters['child_class'] = 'Album'
        if self.channel and self.channel.is_root_node() and not is_paginated:
            filters['show_on_root_channel'] = True
        queryset = Container.objects.filter(
            **filters).exclude(pk__in=self.excluded_ids)

        return queryset._clone()


class AlbumChannelList(ListView):
    model = Album
    type = 'articles'
    template_name_suffix = 'album'

    def get_template_list(self, domain_folder="containers"):
        templates = []
        list_name = 'list'

        if self.template_name_suffix:
            list_fullname = "{0}_{1}".format(self.template_name_suffix,
                                             list_name)

        if self.channel:
            if self.channel.group and self.channel.parent:
                templates.append('{0}/{1}/{2}.html'.format(
                    domain_folder,
                    self.channel.parent.long_slug,
                    list_fullname))

                if self.request.GET.get('page') and\
                   self.__class__.__name__ not in\
                   settings.OPPS_PAGINATE_NOT_APP:
                    templates.append('{0}/{1}/{2}_paginated.html'.format(
                        domain_folder, self.channel.parent.long_slug,
                        list_fullname))

            if self.request.GET.get('page') and\
               self.__class__.__name__ not in settings.OPPS_PAGINATE_NOT_APP:
                templates.append('{0}/{1}/{2}_paginated.html'.format(
                    domain_folder, self.channel.long_slug, list_fullname))

            templates.append('{0}/{1}/{2}.html'.format(
                domain_folder, self.channel.long_slug, list_fullname))

            for t in self.channel.get_ancestors()[::-1]:
                templates.append('{0}/{1}/{2}.html'.format(
                    domain_folder, t.long_slug, list_fullname))
                if self.request.GET.get('page') and\
                   self.__class__.__name__ not in\
                   settings.OPPS_PAGINATE_NOT_APP:
                    templates.append('{0}/{1}/{2}_paginated.html'.format(
                        domain_folder, t.long_slug, list_fullname))

        if self.request.GET.get('page') and\
           self.__class__.__name__ not in settings.OPPS_PAGINATE_NOT_APP:
            templates.append('{0}/{1}_paginated.html'.format(domain_folder,
                                                             list_fullname))

        templates.append('{0}/{1}/{2}.html'.format(
            self.model._meta.app_label,
            self.model._meta.module_name,
            list_name))

        return templates

    def get_template_names(self):
        domain_folder = self.get_template_folder()
        template_list = self.get_template_list(domain_folder)

        return template_list

    def get_queryset(self):
        self.site = get_current_site(self.request)

        queryset = super(AlbumChannelList, self).get_queryset()
        filters = {}
        filters['site_domain'] = self.site.domain
        filters['date_available__lte'] = timezone.now()
        filters['published'] = True
        filters['show_on_root_channel'] = True
        queryset = queryset.filter(**filters)

        return queryset._clone()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.conf.urls import patterns, url
from django.contrib.sitemaps import views as sitemap_views

from opps.core.cache import cache_page

from opps.sitemaps.sitemaps import GenericSitemap, InfoDict
from opps.sitemaps.views import sitemap

sitemaps = {
    'containers': GenericSitemap(InfoDict(), priority=0.6),
}

sitemaps_googlenews = {
    'containers': GenericSitemap(InfoDict(True), priority=0.6),
}

urlpatterns = patterns(
    '',
    url(r'^\.xml$', cache_page(86400)(sitemap_views.index),
        {'sitemaps': sitemaps, 'sitemap_url_name': 'sitemaps'},),
    url(r'^-googlenews\.xml$',
        cache_page(86400)(sitemap),
        {'sitemaps': sitemaps_googlenews,
         'template_name': 'sitemap_googlenews.xml'}),
    url(r'^-(?P<section>.+)\.xml$',
        cache_page(86400)(sitemap_views.sitemap),
        {'sitemaps': sitemaps}, name='sitemaps'),
)



# coding: utf-8
from django.contrib import admin
from django.contrib.auth import get_user_model
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _

from opps.core.admin import apply_opps_rules
from opps.contrib.multisite.admin import AdminViewPermission

from .models import File


@apply_opps_rules('archives')
class FileAdmin(AdminViewPermission):
    search_fields = ['title', 'slug']
    raw_id_fields = ['user']
    list_display = ['title', 'slug', 'download_link', 'published']
    ordering = ('-date_available',)
    list_filter = ['date_available', 'published']
    prepopulated_fields = {"slug": ["title"]}

    fieldsets = (
        (_(u'Identification'), {
            'fields': ('site', 'title', 'slug',)}),
        (_(u'Content'), {
            'fields': ('description', 'archive', 'archive_link', 'tags')}),
        (_(u'Publication'), {
            'classes': ('extrapretty'),
            'fields': ('published', 'date_available',)}),
    )

    def download_link(self, obj):
        html = '<a href="{0}">{1}</a>'.format(
            obj.archive.url,
            unicode(_(u'Download')))
        return html
    download_link.short_description = _(u'download')
    download_link.allow_tags = True

    def save_model(self, request, obj, form, change):
        if not change:
            obj.user = get_user_model().objects.get(pk=request.user.pk)
            obj.date_insert = timezone.now()
        obj.date_update = timezone.now()
        obj.save()

admin.site.register(File, FileAdmin)



from calendar import timegm
from functools import wraps

from django.contrib.sites.models import get_current_site
from django.core.paginator import EmptyPage, PageNotAnInteger
from django.http import Http404
from django.template.response import TemplateResponse
from django.utils import six
from django.utils.http import http_date
from django.conf import settings


def x_robots_tag(func):
    @wraps(func)
    def inner(request, *args, **kwargs):
        response = func(request, *args, **kwargs)
        response['X-Robots-Tag'] = 'noindex, noodp, noarchive'
        return response
    return inner


@x_robots_tag
def sitemap(request, sitemaps, section=None,
            template_name='sitemap.xml', content_type='application/xml'):

    req_protocol = getattr(request, 'scheme', 'http')
    req_site = get_current_site(request)

    if section is not None:
        if section not in sitemaps:
            raise Http404("No sitemap available for section: %r" % section)
        maps = [sitemaps[section]]
    else:
        maps = list(six.itervalues(sitemaps))
    page = request.GET.get("p", 1)

    urls = []
    for site in maps:
        try:
            if callable(site):
                site = site()
            urls.extend(site.get_urls(page=page, site=req_site,
                                      protocol=req_protocol))
        except EmptyPage:
            raise Http404("Page %s empty" % page)
        except PageNotAnInteger:
            raise Http404("No page '%s'" % page)
    context = {
        'urlset': urls,
        'publication_name': getattr(settings, 'OPPS_SITEMAP_PUBLICATION_NAME',
                                    u'Opps CMS'),
        'sitemap_language': getattr(settings, 'OPPS_SITEMAP_LANGUAGE',
                                    u'en')
    }
    response = TemplateResponse(request, template_name, context,
                                content_type=content_type)
    if hasattr(site, 'latest_lastmod'):
        # if latest_lastmod is defined for site, set header so as
        # ConditionalGetMiddleware is able to send 304 NOT MODIFIED
        response['Last-Modified'] = http_date(
            timegm(site.latest_lastmod.utctimetuple()))
    return response



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from unittest import TestCase
from django.template import Template, Context
from django.conf import settings


class TestImagesTagsImageUrl(TestCase):
    url = 'oppsproject.org/path/image.jpg'
    generate_url_path = 'opps.images.templatetags.images_tags.image_url'

    def render(self, arguments):
        source = u'{% load images_tags %}{% image_url ' + arguments + ' %}'
        template = Template(source)
        rendered = template.render(Context({'url': self.url}))
        return rendered.strip()

    def test_templatetag_return(self):
        self.assertTrue(self.render(u'url unsafe=True'))

    def test_should_pass_the_image_url_arg_to_the_helper(self):
        if settings.THUMBOR_ENABLED:
            image_url = (u'http://localhost:8888/unsafe/localhost:8000/media/'
                         u'oppsproject.org/path/image.jpg')
        else:
            image_url = self.url

        self.assertEqual(self.render(u'url unsafe=True'), image_url)

    def test_should_pass_kwargs_to_the_helper(self):
        if settings.THUMBOR_ENABLED:
            image_url = (u'http://localhost:8888/unsafe/300x200/'
                         u'localhost:8000/media/oppsproject.org/path/'
                         u'image.jpg')
        else:
            image_url = self.url

        self.assertEqual(self.render(u'url width=300 height=200 unsafe=True'),
                         image_url)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import json
from django.conf import settings
from django.contrib.syndication.views import Feed
from django.contrib.sites.models import get_current_site
from django.shortcuts import get_object_or_404
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
from django.utils import feedgenerator
from opps.containers.models import Container
from opps.channels.models import Channel


OPPS_FEED_FILTER_DEFAULT = getattr(settings, 'OPPS_FEED_FILTER_DEFAULT', {})
OPPS_FEED_EXCLUDE_DEFAULT = getattr(settings, 'OPPS_FEED_EXCLUDE_DEFAULT', {})


class ItemFeed(Feed):

    feed_type = feedgenerator.Rss201rev2Feed
    description_template = 'articles/feed_item_description.html'
    item_enclosure_length = 1
    item_enclosure_mime_type = "image/jpeg"

    def item_categories(self, obj):
        cats = []
        if obj.channel:
            cats.append(obj.channel.name)
        if getattr(obj, 'tags', None) is not None:
            cats.extend(obj.get_tags() or [])
        return cats

    def item_title(self, item):
        return item.title

    def item_pubdate(self, item):
        return item.date_available

    def item_updateddate(self, item):
        return item.date_update

    def item_link(self, item):
        return item.get_absolute_url()

    def item_enclosure_url(self, item):
        if item.main_image:
            if item.main_image.archive:
                i_url = item.main_image.archive.url
            elif item.main_image.archive_link:
                i_url = item.main_image.archive_link
            else:
                i_url = item.main_image.image_url()

            m_url = getattr(settings, 'MEDIA_URL', '')
            if not m_url.startswith('http') and not i_url.startswith('http'):
                i_url = "http://" + self.site.domain + i_url
            return i_url

    def build_filters(self):
        if not hasattr(self, 'request'):
            return {}

        default = {
            "filter": OPPS_FEED_FILTER_DEFAULT,
            "exclude": OPPS_FEED_EXCLUDE_DEFAULT, }

        data = {"filter": {}, "exclude": {}, }

        r_data = self.request.GET.dict()

        for k, v in r_data.items():
            if k.startswith(('filter', 'exclude')):
                v = json.loads(v)
                for lookup, value in v.items():
                    if lookup.endswith('__in'):
                        v[lookup] = value.split(',')
                data[k].update(v)

        # merges defaults with request.GET data.
        for k, v in data.items():
            data[k] = dict(default[k].items() + v.items())

        return data


class ContainerFeed(ItemFeed):

    link = "/rss"

    def __init__(self, child_class=False):
        self.child_class = child_class

    def __call__(self, request, *args, **kwargs):
        self.site = get_current_site(request)
        self.request = request
        return super(ContainerFeed, self).__call__(request, *args, **kwargs)

    def title(self):
        return _("{0}'s news".format(self.site.name))

    def description(self):
        return _("Latest news on {0}'s".format(self.site.name))

    def items(self):
        container = Container.objects.filter(
            site=self.site,
            date_available__lte=timezone.now(),
            published=True,
            channel__include_in_main_rss=True,
            channel__published=True
        )

        if self.child_class:
            container = container.filter(child_class=self.child_class)

        container = container.exclude(child_class__in=['Mirror', 'Entry'])

        return container.order_by('-date_available')[:40]


class ChannelFeed(ItemFeed):
    """
    Items can be filtered using "filter" and "exclude" querystring args.
    examples:

    - get only entries with images
    rss?filter={"main_image__isnull": false}

    - exclude specific child_class
    rss?exclude={"child_class__in": "Album,Poll"}

    The format is json
    """

    def get_object(self, request, long_slug):
        self.site = get_current_site(request)
        self.request = request
        channel = get_object_or_404(Channel,
                                    site=self.site,
                                    long_slug=long_slug)
        self.channel_descendants = channel.get_descendants(include_self=True)
        return channel

    def link(self, obj):
        return _("{0}RSS".format(obj.get_absolute_url()))

    def title(self, obj):
        return _(u"{0}'s news on channel {1}".format(self.site.name,
                                                     obj.name))

    def description(self, obj):
        return _(u"Latest news on {0}'s channel {1}".format(self.site.name,
                                                            obj.name))

    def items(self, obj):
        filters = self.build_filters().get('filter', {})
        excludes = self.build_filters().get('exclude', {})

        channel_long_slugs = [
            children.long_slug for children in
            self.channel_descendants
        ]

        qs = Container.objects.filter(
            site=self.site,
            channel_long_slug__in=channel_long_slugs,
            date_available__lte=timezone.now(),
            published=True,
            **filters
        ).exclude(
            child_class__in=['Mirror', 'Entry'],
        ).exclude(
            **excludes
        )

        qs = qs.order_by(
            '-date_available'
        ).select_related('publisher')[:40]

        return qs


class ContainerAtomFeed(ContainerFeed):
    link = "/atom"
    feed_type = feedgenerator.Atom1Feed


class ChannelAtomFeed(ChannelFeed):
    feed_type = feedgenerator.Atom1Feed



# -*- coding: utf-8 -*-
import pkg_resources


pkg_resources.declare_namespace(__name__)

VERSION = (0, 2, 7, 1)
__version__ = ".".join(map(str, VERSION))
__status__ = "Development"
__description__ = u"Open Source Content Management Platform - CMS for the "
u"magazines, newspappers websites and portals with "
u"high-traffic, using the Django Framework."

__author__ = u"Thiago Avelino"
__credits__ = ['Bruno Rocha']
__email__ = u"thiago@avelino.xxx"
__license__ = u"MIT"
__copyright__ = u"Copyright 2014, Thiago Avelino"



# -*- coding: utf-8 -*-
from collections import Counter
import logging

from django import template
from django.conf import settings
from django.utils import timezone
from django.utils.safestring import mark_safe
from django.template.defaultfilters import linebreaksbr
from django.core.cache import cache
from django.contrib.sites.models import Site

from opps.channels.models import Channel
from opps.contrib.middleware.global_request import get_request
from opps.containers.models import Container, ContainerBox, Mirror

from magicdate import magicdate


register = template.Library()
logger = logging.getLogger()


@register.assignment_tag
def get_tags_counter(queryset=None, n=None):
    if queryset is None:
        queryset = Container.objects.all_published()

    counter = Counter()
    qs = queryset.filter(tags__isnull=False).exclude(tags="").order_by()
    print qs.count()
    for tags in qs.values_list("tags", flat=True).distinct():
        l = [i.strip() for i in tags.split(",") if i.strip()]
        counter.update(l)

    return counter.most_common(n)


@register.filter
def values_list_flat(queryset, field='pk'):
    return queryset.values_list(field, flat=True)


@register.assignment_tag
def get_recommendations(query_slice, child_class, container):
    """
    Takes the container object and get recommendations and filters by given
    child_class and limit the result by given slice.

    Sample usage::

        {% get_recommendations ":3" "post" container as context_var %}

    query_slice:
        A string with slice notation to limit the queryset result
    child_class:
        Name of child class
    container:
        A Container object
    """

    if not query_slice:
        query_slice = ":"

    bits = []
    for x in query_slice.split(':'):
        if len(x) == 0:
            bits.append(None)
        else:
            bits.append(int(x))

    return container.recommendation(child_class, bits)


@register.assignment_tag(takes_context=True)
def load_boxes(context, slugs=None, **filters):
    if slugs:
        filters['slug__in'] = ordered_slugs = slugs.split(',')

    request = context['request']
    current_site = getattr(
        request,
        'site',
        Site.objects.get(pk=settings.SITE_ID)
    )

    filters['site__in'] = [current_site]
    master_site = settings.OPPS_CONTAINERS_SITE_ID or 1
    if current_site.id != master_site:
        filters['site__in'].append(master_site)

    filters['date_available__lte'] = timezone.now()
    filters['published'] = True

    boxes = ContainerBox.objects.filter(**filters).order_by('-site')
    fallback = getattr(settings, 'OPPS_MULTISITE_FALLBACK', False)

    exclude_ids = []

    if slugs:
        ob = lambda i, o=ordered_slugs: (
            i.site != current_site, i.site.id, o.index(i.slug)
        )

        boxes = sorted(
            boxes,
            key=ob
        )

    for box in boxes:
        if box.queryset:
            results = box.get_queryset(exclude_ids=exclude_ids)
        else:
            results = box.ordered_containers(exclude_ids=exclude_ids)

        if box.queryset:
            [exclude_ids.append(i.pk)
             for i in results if i.pk not in exclude_ids
             and issubclass(i.__class__, Container)]
        elif fallback:
            [exclude_ids.append(i.container_id)
             for i in results
             if i.container_id and i.container_id not in exclude_ids]
        else:
            [exclude_ids.append(i.pk)
             for i in results
             if i.pk not in exclude_ids]

    results = {}
    for box in boxes:
        if box.slug not in results:
            results[box.slug] = box

    get_request().container_boxes = results
    return results


@register.simple_tag(takes_context=True)
def get_containerbox(
        context, slug, template_name=None, channel=None, **extra_context):

    request = context['request']
    current_site = getattr(
        request,
        'site',
        Site.objects.get(pk=settings.SITE_ID)
    )
    is_mobile = getattr(request, 'is_mobile', False)

    cachekey = "ContainerBox-{0}-{1}-{2}-{3}".format(
        slug,
        template_name,
        is_mobile,
        current_site.id
    )

    render = cache.get(cachekey)
    if render:
        return render

    box = getattr(get_request(), 'container_boxes', {}).get(slug, None)

    if not box:
        filters = {}
        filters['site'] = current_site
        filters['slug'] = slug
        filters['date_available__lte'] = timezone.now()
        filters['published'] = True
        if channel is not None:
            filters['channel'] = channel

        master_site = settings.OPPS_CONTAINERS_SITE_ID or 1

        try:
            box = ContainerBox.objects.get(**filters)
        except ContainerBox.DoesNotExist:
            box = None

        if current_site.id != master_site and \
           not box or not getattr(box, 'has_content', False):
            filters['site'] = master_site
            try:
                box = ContainerBox.objects.get(**filters)
            except ContainerBox.DoesNotExist:
                box = None

        if not box:
            box = ContainerBox.objects.none()

    t = template.loader.get_template('articles/articlebox_detail.html')
    if template_name:
        t = template.loader.get_template(template_name)

    context = {
        'articlebox': box,
        'slug': slug,
        'context': context,
        'request': request
    }

    context.update(extra_context)

    render = t.render(template.Context(context))

    cache.set(cachekey, render, settings.OPPS_CACHE_EXPIRE)

    return render


@register.simple_tag
def get_all_containerbox(channel_long_slug=None, template_name=None):
    """
    Takes all containers or containers that match the channel name (long slug).

    Sample usages::

        {% get_all_containerbox "channel" template_name='my_template.html' %}
        {% get_all_containerbox "channel/subchannel" %}
        {% get_all_containerbox %}

    channel_long_slug:
        Long path to channel (including subchannel if is the case)
    """

    cachekey = "get_all_containerbox-{0}-{1}".format(
        channel_long_slug,
        template_name)

    render = cache.get(cachekey)
    if render:
        return render

    filters = {}
    filters['date_available__lte'] = timezone.now()
    filters['published'] = True
    filters['site'] = settings.SITE_ID
    if settings.OPPS_CONTAINERS_SITE_ID:
        filters['site'] = settings.OPPS_CONTAINERS_SITE_ID

    boxes = ContainerBox.objects.filter(**filters)

    if channel_long_slug:
        boxes = boxes.filter(channel_long_slug=channel_long_slug)

    t = template.loader.get_template('articles/articlebox_list.html')
    if template_name:
        t = template.loader.get_template(template_name)

    render = t.render(template.Context({'articleboxes': boxes}))
    cache.set(cachekey, render, settings.OPPS_CACHE_EXPIRE)

    return render


@register.simple_tag
def get_post_content(post, template_name='containers/post_related.html',
                     content_field='content', related_name='related_posts',
                     get_related=True, safe=True, divider="<br />",
                     placeholder=settings.OPPS_RELATED_POSTS_PLACEHOLDER):
    """
    takes the post and tries to find the related posts to embed inside
    the content, if not found return only the content.

    post:
        Post instance
    template_name:
        path to template which receives the related posts
    content_field:
        name of the field with post content
    related_name:
        a m2m field name or a @property name which
        returns a queryset of related posts
    get_related:
        if False bypass and return only the content
    safe:
        if True mark the content as safe
    divider:
        used when there is no placeholder
    placeholder:
        the string to replace ex: --related--
    """
    if not hasattr(post, content_field):
        return None
    content = getattr(post, content_field, '')

    # REMOVE NEW LINES
    content = linebreaksbr(content)

    # Fix embed allowfullscreen
    # TinyMCE BUG
    content = content.replace('allowfullscreen="allowfullscreen"',
                              'allowfullscreen="true"')

    if not get_related:
        return content

    related_posts = getattr(post, related_name, None)

    if not related_posts.exists():
        return mark_safe(content)

    # GET THE TEMPLATE
    t = template.loader.get_template(template_name)
    related_rendered = t.render(template.Context({
        'post': post, related_name: related_posts}))
    # EMBED RELATED POSTS
    if placeholder in content:
        return mark_safe(content.replace(
            placeholder,
            related_rendered
        ))
    else:
        return mark_safe(content + divider + related_rendered)


@register.simple_tag
def get_url(obj, http=False, target=None, url_only=False):

    if not hasattr(obj, 'child_class'):
        return obj.get_absolute_url()

    try:
        _url = obj.get_absolute_url()
        _target = target or '_self'
        _is_link = obj.child_class == 'Link'
        # Determine if it's a local or foreign link
        if _is_link and not obj.link.is_local() and not target:
            _target = '_blank'
        # Determine url type
        if http:
            _url = 'http://{0}{1}'.format(
                obj.site,
                obj.get_absolute_url())
        if url_only:
            return _url
        return 'href="{0}" target="{1}"'.format(_url, _target)
    except Exception as e:
        logger.error("Exception at templatetag get_url: {0}".format(e))
        return obj.get_absolute_url()


@register.assignment_tag
def get_containers_by(limit=None, **filters):
    """Return a list of containers filtered by given args"""
    cachekey = u'getcontainersby-{0}'.format(hash(frozenset(filters.items())))
    _cache = cache.get(cachekey)
    if _cache:
        return _cache

    site = settings.SITE_ID
    if settings.OPPS_CONTAINERS_SITE_ID:
        site = settings.OPPS_CONTAINERS_SITE_ID

    qs = Container.objects.all_published()
    qs = qs.filter(site=site, **filters)
    qs = qs[:limit]
    containers = [i for i in qs]

    cache.set(cachekey, containers, 3600)
    return containers


@register.assignment_tag
def filter_queryset_by(queryset, **filters):
    """Filter object list"""
    if not getattr(queryset, 'query', False):
        return queryset

    cachekey = u'filterquerysetby-{0}'.format(hash(unicode(queryset.query)))
    _cache = cache.get(cachekey)
    if _cache:
        return _cache

    # check for __in lookups and split the param
    found_in_lookup = None
    for key in filters.keys():
        if '__in' in key:
            found_in_lookup = key

    if found_in_lookup:
        filters[found_in_lookup] = filters[found_in_lookup].split(',')

    if not queryset.query.can_filter():
        # create new queryset based on the ids and apply filter
        ids = [i.id for i in queryset]
        queryset = queryset.model.objects.filter(id__in=ids).filter(**filters)
        return queryset

    containers = queryset.filter(**filters)
    cache.set(cachekey, containers, 3600)
    return containers


@register.assignment_tag
def exclude_queryset_by(queryset, **excludes):
    """Exclude object list"""

    if not getattr(queryset, 'query', False):
        return queryset

    cachekey = u'excludequerysetby-{0}'.format(hash(unicode(queryset.query)))
    _cache = cache.get(cachekey)
    if _cache:
        return _cache

    # check for __in lookups and split the param
    found_in_lookup = None
    for key in excludes.keys():
        if '__in' in key:
            found_in_lookup = key

    if found_in_lookup:
        excludes[found_in_lookup] = excludes[found_in_lookup].split(',')

    if not queryset.query.can_filter():
        # create new queryset based on the ids and apply filter
        ids = [i.id for i in queryset]
        containers = queryset.model.objects.filter(id__in=ids).exclude(
            **excludes
        )
    else:
        containers = queryset.exclude(**excludes)

    if 'child_class' in excludes:
        # we need to exclude the mirrors containing the child_class that
        # we want to exclude
        bad_child_class = excludes['child_class']
        mirrors = Mirror.objects.filter(
            container__child_class=bad_child_class
        ).values_list('id', flat=True)
        if mirrors:
            containers = containers.exclude(pk__in=mirrors)

    cache.set(cachekey, containers, 3600)
    return containers


@register.assignment_tag
def get_container_by_channel(slug, number=10, depth=1,
                             include_children=True, **kwargs):
    box = None
    magic_date = kwargs.pop('magic_date', False)
    date = timezone.now()

    if magic_date:
        try:
            date = magicdate(magic_date)
        except Exception:
            pass

    # __in split treatment
    splited = dict([
        (key, value.split(','))
        for key, value
        in kwargs.items()
        if key.endswith('__in') and type(value) is not list])
    kwargs.update(splited)

    if include_children:
        k = 'channel_id__in'
        kwargs[k] = cache.get(
            'get_container_by_channel-{0}'.format(slug))
        if not kwargs[k]:

            try:
                channel = Channel.objects.get(long_slug=slug)
                qs = channel.get_descendants(include_self=True)
                qs = qs.filter(level__lte=channel.level + depth)
                kwargs[k] = \
                    qs.values_list("id", flat=True)
                cache.set(
                    'get_container_by_channel-{0}'.format(slug),
                    kwargs[k],
                    settings.OPPS_CACHE_EXPIRE)

            except Channel.DoesNotExist:
                kwargs[k] = []

    try:
        kwargs['site'] = settings.SITE_ID
        if settings.OPPS_CONTAINERS_SITE_ID:
            kwargs['site'] = settings.OPPS_CONTAINERS_SITE_ID
        kwargs['show_on_root_channel'] = include_children
        kwargs['date_available__lte'] = date
        kwargs['published'] = True
        box = Container.objects.distinct().filter(
            **kwargs).order_by('-date_available')[:number]
    except:
        pass
    return box


@register.assignment_tag
def get_containerbox_by(**filters):
    """Return a list of containers filtered by given args"""
    site = settings.SITE_ID
    if settings.OPPS_CONTAINERS_SITE_ID:
        site = settings.OPPS_CONTAINERS_SITE_ID
    return ContainerBox.objects.filter(site=site,
                                       published=True,
                                       date_available__lte=timezone.now(),
                                       **filters)


@register.simple_tag(takes_context=True)
def get_containerbox_list(context, slug, num=0, template_name=None):
    """ returns a list of sub-lists of the containerbox specific containers,
        the size of the sub lists is treated with a parameter num """

    request = context['request']

    cachekey = "ContainerBoxList-{0}-{1}-{2}".format(
        slug,
        template_name,
        request.is_mobile,
    )

    render = cache.get(cachekey)
    if render:
        return render

    site = settings.SITE_ID
    if settings.OPPS_CONTAINERS_SITE_ID:
        site = settings.OPPS_CONTAINERS_SITE_ID
    try:
        box = ContainerBox.objects.filter(
            site=site, slug=slug,
            date_available__lte=timezone.now(),
            published=True)
        if isinstance(num, int) and num > 0 and box:
            list_box = box[0].ordered_box_containers()
            box = [list_box[i:i + num] for i in range(0, len(list_box), num)]
    except ContainerBox.DoesNotExist:
        box = None

    t = template.loader.get_template('articles/articlebox_container_list.html')
    if template_name:
        t = template.loader.get_template(template_name)

    render = t.render(template.Context({
        'list_container': box,
        'slug': slug,
        'context': context}
    ))

    cache.set(cachekey, render, settings.OPPS_CACHE_EXPIRE)

    return render


@register.assignment_tag
def get_custom_field_value(obj, field_slug):
    """
    Return a custom field value
    """
    if not callable(getattr(obj, 'custom_fields')):
        return None

    if not obj.custom_fields():
        return None

    return obj.custom_fields().get(field_slug)


@register.assignment_tag
def get_postrelated_by(obj, **filters):
    """Return a list of post related filtered by given args"""
    if getattr(obj, 'postrelated_post', False):
        cachekey = u'getpostrelatedby-{0}-{1}'.format(
            hash(frozenset(filters.items())), obj.pk)

        _cache = cache.get(cachekey)

        if _cache:
            return _cache

        queryset = obj.postrelated_post.filter(post__pk=obj.pk)

        if 'exclude' in filters.keys():
            del filters['exclude']
            containers = [i.related for i in queryset.exclude(**filters)
                                                     .order_by('order')]
        else:
            containers = [i.related for i in queryset.filter(**filters)
                                                     .order_by('order')]

        cache.set(cachekey, containers, 3600)
        return containers
    return ''



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.contrib.auth import get_user_model
from django.contrib.sites.models import Site

from opps.api import BaseHandler

from .models import Logging


class LoggingHandler(BaseHandler):
    allowed_methods = ('POST', 'GET')
    model = Logging

    def create(self, request):
        method = getattr(request, request.method)

        User = get_user_model()
        username = method.get('api_username')
        try:
            user = User.objects.get(username=username)
        except User.DoesNotExist:
            return {}

        try:
            site = Site.objects.get(domain=method.get('site'))
        except Site.DoesNotExist:
            site = Site.objects.order_by('id')[0]

        log = Logging.objects.create(
            user=user,
            site=site,
            application=method.get('application'),
            action=method.get('action'),
            text=method.get('text'),
        )
        return log
